package com.wechat.common.action;
import org.beetl.sql.core.engine.PageQuery;
import org.springframework.web.bind.annotation.ModelAttribute;
import com.wechat.common.parameter.ProjectConstant;
import com.wechat.common.request.RequestUtil;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.util.Map;

/**
 * 基础  controller 权限获取
 * 在每个controller继承该类
 */
public class BaseController {
    protected HttpServletRequest request;
    protected HttpServletResponse response;
    
    
    /**
     * 表示请求该类的每个Action前都会首先执行它 setReqAndRes
     * @param request
     * @param response
     */
    @ModelAttribute
    public void setReqAndRes(HttpServletRequest request, HttpServletResponse response) {
        this.request = request;
        this.response = response;
        
    }
    
 
    
    /**
     * 分页参数信息
     * @return
     */
    protected PageQuery<Map<String,Object>> pageQuery(){
        PageQuery<Map<String,Object>> page = new PageQuery<>();
        //设置分页信息
        page.setPageNumber(RequestUtil.getIntParam(request,"pageNumber",1));
        page.setPageSize(RequestUtil.getIntParam(request,"pageSize",99999));
        //设置排序规则，如果不传，默认第一列排序
        page.setOrderBy(RequestUtil.getStringParam(request,"orderBy","1 DESC"));
        return page;
    }
    /**
     * 获取客服端ip
     * X-Forwarded-For(XFF)是用来识别通过HTTP代理或负载均衡方式连接到Web服务器的客户端最原始的IP地址的HTTP请求头字段
     * @return
     */
    protected String getClientAddress() {
        String ip = request.getHeader(ProjectConstant.ProjectIpAddress.IP_FORWARDED);
        if (ip == null || ip.length() == 0 || ProjectConstant.ProjectIpAddress.UN_KNOWN.equalsIgnoreCase(ip)) {
            ip = request.getHeader(ProjectConstant.ProjectIpAddress.IP_PROXY_CLIENT);
        }
        if (ip == null || ip.length() == 0 || ProjectConstant.ProjectIpAddress.UN_KNOWN.equalsIgnoreCase(ip)) {
            ip = request.getHeader(ProjectConstant.ProjectIpAddress.IP_WL_FORWARDED);
        }
        if (ip == null || ip.length() == 0 || ProjectConstant.ProjectIpAddress.UN_KNOWN.equalsIgnoreCase(ip)) {
            ip = request.getRemoteAddr();
        }
        ip = ip.equals(ProjectConstant.ProjectIpAddress.DEFAULT_IP) ? ProjectConstant.ProjectIpAddress.LOCAL_IP : ip;
        int index = ip.indexOf(",");
        if (index != -1) ip = ip.substring(0, index);
        return ip;
    }
}
