package com.wechat.common.entity.merchant.bank_account_info;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.security.cert.X509Certificate;

import javax.crypto.IllegalBlockSizeException;

import com.wechat.common.entity.merchant.WeChatMerchant;
import com.wechat.common.entity.merchant.RegisterMerchant.RegisterMerchant;
import com.wechat.common.util.StringUtil;
import com.wechat.common.util.serial.WechatpaySerial;

public class BankAccountInfo {
	
	/**
	 * 1、若主体为企业/党政、机关及事业单位/其他组织，可填写：对公银行账户。
		2、若主体为个体户，可选择填写：对公银行账户或经营者个人银行卡。
		枚举值：

			BANK_ACCOUNT_TYPE_CORPORATE：对公银行账户
			BANK_ACCOUNT_TYPE_PERSONAL：经营者个人银行卡
			示例值：BANK_ACCOUNT_TYPE_CORPORATE
	 */
	private String bank_account_type;
	
	/**
	 * 1、选择“经营者个人银行卡”时，开户名称必须与“经营者证件姓名”一致。
		2、选择“对公银行账户”时，开户名称必须与营业执照上的“商户名称”一致。
		3、该字段需进行加密处理，加密方法详见敏感信息加密说明。
		示例值：AOZdYGISxo4y4+kyblK2FCCqQJJp0/0eow96uY1Pk7Rq79Jtt7+I8juwEc4P4TG5xzchG/5IL9DBd+Z0zZXkw==
	 */
	private String account_name;
	
	/**
	 * 开户银行，详细参见开户银行对照表。
		示例值：工商银行
	 */
	private String account_bank;
	
	/**
	 * 至少精确到市，详细参见省市区编号对照表。
		示例值：110000
	 */
	private String bank_address_code;
	
	/**
	 * 1、17家直连银行无需填写，如为其他银行，则开户银行全称（含支行）和开户银行联行号二选一。
		2、详细参见开户银行全称（含支行）对照表。
		示例值：402713354941
	 */
	private String bank_branch_id;
	
	/**
	 * 1、17家直连银行无需填写，如为其他银行，则开户银行全称（含支行）和 开户银行联行号二选一。
		2、需填写银行全称，如"深圳农村商业银行XXX支行"，详细参见开户银行全称（含支行）对照表。
		示例值：施秉县农村信用合作联社城关信用社
	 */
	private String bank_name;
	
	/**
	 * 1、数字，长度遵循系统支持的卡号长度要求表。
		2、该字段需进行加密处理，加密方法详见敏感信息加密说明。 
		示例值：d+xT+MQCvrLHUVDWC1PHN4C7Rsu3WL8sPndjXTd75kPkyjqnoMRrEEaYQE8ZRGYoeorwC+w==
	 */
	private String account_number;

	public String getBank_account_type() {
		return bank_account_type;
	}

	public void setBank_account_type(String bank_account_type) {
		this.bank_account_type = bank_account_type;
	}

	public String getAccount_name() {
		return account_name;
	}

	public void setAccount_name(String account_name) {
		this.account_name = account_name;
	}

	public String getAccount_bank() {
		return account_bank;
	}

	public void setAccount_bank(String account_bank) {
		this.account_bank = account_bank;
	}

	public String getBank_address_code() {
		return bank_address_code;
	}

	public void setBank_address_code(String bank_address_code) {
		this.bank_address_code = bank_address_code;
	}

	public String getBank_branch_id() {
		return bank_branch_id;
	}

	public void setBank_branch_id(String bank_branch_id) {
		this.bank_branch_id = bank_branch_id;
	}

	public String getBank_name() {
		return bank_name;
	}

	public void setBank_name(String bank_name) {
		this.bank_name = bank_name;
	}

	public String getAccount_number() {
		return account_number;
	}

	public void setAccount_number(String account_number) {
		this.account_number = account_number;
	}
	
	/**
	 * 
	 * @param certificate
	 */
	public void encryption(X509Certificate certificate) {
		try {
			this.account_name = WechatpaySerial.rsaEncryptOAEP(this.account_name, certificate);
			this.account_number = WechatpaySerial.rsaEncryptOAEP(this.account_number, certificate);
		} catch (IllegalBlockSizeException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void setInfo(RegisterMerchant registerMerchant,X509Certificate certificate, WeChatMerchant wm) {
		this.setBank_account_type(registerMerchant.getBankAccountType());
		this.setAccount_name(registerMerchant.getAccountName());
		this.setBank_address_code(registerMerchant.getBankAddressCode());
		this.setBank_branch_id(registerMerchant.getBankBranchId());
		try {
			this.setAccount_bank(URLDecoder.decode(registerMerchant.getAccountBank(),"UTF-8"));
			if(!StringUtil.isEmpty(registerMerchant.getBankName())){
				this.setBank_name(URLDecoder.decode(registerMerchant.getBankName(),"UTF-8"));
			}
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		this.setAccount_number(registerMerchant.getAccountNumber());
		this.encryption(certificate);
		wm.setBank_account_info(this);
	}
	
	
	
	
}
