package com.wechat.common.util;

import java.io.InputStream;
import java.security.KeyStore;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.net.ssl.SSLContext;

import com.wechat.common.parameter.ProjectConstant;
import org.apache.http.HttpEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.conn.ssl.SSLContexts;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.io.ClassPathResource;


/**
 * This example demonstrates how to create secure connections with a custom SSL
 * context.
 */
public class ComtikaHttpsUtil {
    private static Logger logger = LoggerFactory.getLogger(ComtikaHttpsUtil.class);
    private static final Map<String,CloseableHttpClient> HTTPS_MAP = new ConcurrentHashMap<>(4);

    public static void stop(){
        HTTPS_MAP.forEach((k,v)->{
            try {
                v.close();
            }catch (Exception e){}

        });
    }
    // 这里是其他方式post 请求
    public static String doPost(String url,String data)throws Exception{
        return doCertWX(url,data,null);
    }



    public static String doCertWX(String url, String data) throws Exception{
        return doCertWX(url,data, ProjectConstant.merchantId);
    }
    private static String KTJ_DEFAULT_kEY = "KTJ_DEFAULT_kEY";
    private static CloseableHttpClient findHttpClient(String mchId){
        if(StringUtil.isEmpty(mchId)){// 这里不用使用默认的http
            CloseableHttpClient closeableHttpClient = HTTPS_MAP.get(KTJ_DEFAULT_kEY);
            if(closeableHttpClient == null){
                closeableHttpClient = HttpClients.createDefault();
                HTTPS_MAP.put(KTJ_DEFAULT_kEY,closeableHttpClient);
            }
            return closeableHttpClient;
        }
        CloseableHttpClient closeableHttpClient = HTTPS_MAP.get(mchId);
        if(closeableHttpClient == null){
            /**
             * 注意PKCS12证书 是从微信商户平台-》账户设置-》 API安全 中下载的
             */
            try {
                ClassPathResource cp = new ClassPathResource("apiclient_cert.p12");
                InputStream inStream = cp.getInputStream();
                KeyStore keyStore = KeyStore.getInstance("PKCS12");
                try {
                    keyStore.load(inStream, mchId.toCharArray());//这里写密码..默认是你的MCHID
                } finally {
                    inStream.close();
                }
                // Trust own CA and all self-signed certs
                /**
                 * 此处要改
                 * */
                SSLContext sslcontext = SSLContexts.custom()
                        .loadKeyMaterial(keyStore, mchId.toCharArray())//这里也是写密码的
                        .build();
                // Allow TLSv1 protocol only
                SSLConnectionSocketFactory sslsf = new SSLConnectionSocketFactory(
                        sslcontext,
                        new String[]{"TLSv1"},
                        null,
                        SSLConnectionSocketFactory.BROWSER_COMPATIBLE_HOSTNAME_VERIFIER);
                closeableHttpClient = HttpClients.custom()
                        .setSSLSocketFactory(sslsf)
                        .build();
                HTTPS_MAP.put(mchId,closeableHttpClient);
            }catch (Exception e){
                logger.error("初始化SSL错误：{}",e);
                throw new RuntimeException("初始化HTTP出错:"+e.getMessage());
            }
        }
        return closeableHttpClient;
    }

    public static String doCertWX(String url, String data,String mchId) throws Exception {
        CloseableHttpClient httpclient = findHttpClient(mchId);
        HttpPost httpost = new HttpPost(url); // 设置响应头信息
        //httpost.addHeader("Connection", "keep-alive");
        //httpost.addHeader("Accept", "*/*");
        //httpost.addHeader("Content-Type", "application/x-www-form-urlencoded; charset=UTF-8");
        // httpost.addHeader("Host", "api.mch.weixin.qq.com");
        // httpost.addHeader("X-Requested-With", "XMLHttpRequest");
        //httpost.addHeader("Cache-Control", "max-age=0");
        // httpost.addHeader("User-Agent", "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0) ");
        httpost.setEntity(new StringEntity(data, "UTF-8"));
        CloseableHttpResponse response = httpclient.execute(httpost);
        try {
            HttpEntity entity = response.getEntity();
            String jsonStr = EntityUtils.toString(response.getEntity(), "UTF-8");
            EntityUtils.consume(entity);
            return jsonStr;
        } finally {
            response.close();
        }
    }
}
