package com.wechat.common.util;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.security.KeyFactory;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.security.cert.Certificate;
import java.security.cert.CertificateFactory;
import java.security.spec.PKCS8EncodedKeySpec;
import java.util.List;
import java.util.Map;
import java.util.UUID;


import com.alibaba.fastjson.JSONObject;
import com.wechat.common.response.RespWrapper;
import com.wechat.common.util.ComtikaWeChatUtil;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.digest.DigestUtils;

import com.wechat.common.parameter.ProjectConstant;

public class WeChatUploadUtil {
    // 换行符
    private static final String LINE_END = "\r\n";
    private static final String PREFIX = "--";
    private static final String BOUNDARY = "duojil";
    private static final String FILE_META = "{\"filename\":\"%s\",\"sha256\":\"%s\"}";

    public static void main(String[] args)throws Exception{
        File file = new File("G:/2222.jpg");
        FileInputStream inputStream = new FileInputStream(file);
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        byte[] buffer = new byte[4096];
        int n = 0;
        while (-1 != (n = inputStream.read(buffer))) {
            output.write(buffer, 0, n);
        }
        doWeChantPoto(output.toByteArray(),"测试使用.jpg");
    }

    public static RespWrapper<String> doWeChantPoto(byte[] bytes,String fileName)throws Exception{
        ByteArrayInputStream inputStream = new ByteArrayInputStream(bytes);
        String fileSha256 = DigestUtils.sha256Hex(inputStream);//文件sha256值
        String body = String.format(FILE_META, fileName, fileSha256);
        String authorization = ComtikaWeChatUtil.findAuthorization("POST", "/v3/merchant/media/upload", body);
        URL url = new URL("https://api.mch.weixin.qq.com/v3/merchant/media/upload");
        HttpURLConnection conn = (HttpURLConnection) url.openConnection();
        // 设置为POST
        conn.setRequestMethod("POST");
        // 发送POST请求必须设置如下两行
        conn.setDoOutput(true);
        conn.setDoInput(true);
        conn.setUseCaches(false);
        // 设置请求头参数
        conn.setRequestProperty("Charsert", "UTF-8");
        conn.setRequestProperty("Accept","application/json");
        conn.setRequestProperty("Content-Type", "multipart/form-data; boundary=" + BOUNDARY);
        conn.setRequestProperty("Authorization", authorization);

        DataOutputStream dos = new DataOutputStream(conn.getOutputStream());

        //拼装请求内容第一部分
        StringBuilder strSb = new StringBuilder();
        strSb.append(PREFIX).append(BOUNDARY).append(LINE_END)
                .append("Content-Disposition: form-data; name=\"meta\";" + LINE_END)
                .append("Content-Type: application/json; " + LINE_END)
                .append(LINE_END)// 空行
                .append(body)
                .append(LINE_END);
        dos.write(strSb.toString().getBytes());

        dos.flush();

        //拼装请求内容第二部分
        StringBuilder fileSbStart = new StringBuilder();
        fileSbStart.append(PREFIX).append(BOUNDARY).append(LINE_END)
                .append("Content-Disposition: form-data; name=\"file\"; filename=\""+ fileName+ "\";" + LINE_END)
                .append("Content-Type: image/jpeg" + LINE_END)
                .append(LINE_END);// 空行
        dos.write(fileSbStart.toString().getBytes());

        dos.flush();

        //文件二进制内容
        // InputStream is = new FileInputStream(file);
        // byte[] buffer = new byte[1024];
        // int len = 0;
        // while ((len = is.read(buffer)) != -1){
        //     dos.write(buffer,0,len);
        // }
        // is.close();
        dos.write(bytes);

        //拼装请求内容结尾
        StringBuilder fileSbEnd = new StringBuilder();
        fileSbEnd.append(LINE_END)
                .append(PREFIX).append(BOUNDARY).append(PREFIX)
                .append(LINE_END);

        dos.write(fileSbEnd.toString().getBytes());

        dos.flush();
        dos.close();

        //接收返回
        //打印返回头信息
        // System.out.println("接口返回头信息:");
        // Map<String, List<String>> responseHeader = conn.getHeaderFields();
        // for (Map.Entry<String, List<String>> entry : responseHeader.entrySet()) {
        //     System.out.println(entry.getKey()+":" + entry.getValue());
        // }

        //打印返回内容
        int responseCode = conn.getResponseCode();
        String rescontent = "";
        if((responseCode+"").startsWith("2")){
            //成功
            rescontent = new String(InputStreamTOByte(conn.getInputStream()));
            System.out.println("图片上传成功:"+rescontent);
        }else{
            //失败
            rescontent = new String(InputStreamTOByte(conn.getErrorStream()));
            System.out.println("图片上传失败:"+rescontent);
        }
        //验证微信支付返回签名
        //String Wtimestamp = responseHeader.get("Wechatpay-Timestamp").get(0);
        //String Wnonce = responseHeader.get("Wechatpay-Nonce").get(0);
        //String Wsign = responseHeader.get("Wechatpay-Signature").get(0);
        ////拼装待签名串
        //StringBuffer ss = new StringBuffer();
        //ss.append(Wtimestamp).append("\n");
        //ss.append(Wnonce).append("\n");
        //ss.append(rescontent).append("\n");

        JSONObject jsonObject = JSONObject.parseObject(rescontent);
        // 校验成功 直接返回
        // {"media_id":"m88FY750V8XYG12e_DqR50H4okiTa3SuHFV-RVUeDbXGChlyp7mvivJb7p-yPxrzhp71150CxPRqa4-Qrr9mubtnts98dtiVlr7tRyd3shY"}
        String media_id = jsonObject.getString("media_id");
        if(media_id != null)
            return RespWrapper.makeSuccessResp(media_id);
        return RespWrapper.makeFailResp(rescontent);

    }



	public static byte[] InputStreamTOByte(InputStream in) throws IOException{

        int BUFFER_SIZE = 4096;
        ByteArrayOutputStream outStream = new ByteArrayOutputStream();
        byte[] data = new byte[BUFFER_SIZE];
        int count = -1;
        while((count = in.read(data,0,BUFFER_SIZE)) != -1)
            outStream.write(data, 0, count);
        data = null;
        byte[] outByte = outStream.toByteArray();
        outStream.close();

        return outByte;
    }
	public static byte[] signRSA(String data, String priKey) throws Exception {
		//签名的类型
		Signature sign = Signature.getInstance("SHA256withRSA");
		//读取商户私钥,该方法传入商户私钥证书的内容即可
		byte[] keyBytes = Base64.decodeBase64(priKey);
		PKCS8EncodedKeySpec keySpec = new PKCS8EncodedKeySpec(keyBytes);
		KeyFactory keyFactory = KeyFactory.getInstance("RSA");
		PrivateKey privateKey = keyFactory.generatePrivate(keySpec);
		sign.initSign(privateKey);
		sign.update(data.getBytes("UTF-8"));
		return sign.sign();
	}
	 public static boolean verifyRSA(String data, byte[] sign, String pubKey) throws Exception{
		 if(data == null || sign == null || pubKey == null){
			 return false;
		 }
		 CertificateFactory cf = CertificateFactory.getInstance("X.509");
	     FileInputStream in = new FileInputStream(pubKey);
	     Certificate c = cf.generateCertificate(in);
	     in.close();
	     PublicKey publicKey = c.getPublicKey();
	     Signature signature = Signature.getInstance("SHA256WithRSA");
	     signature.initVerify(publicKey);
	     signature.update(data.getBytes("UTF-8"));
	     return signature.verify(sign);
	 }
}
