package com.wechat.common.util;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.util.ArrayList;
import java.util.Base64;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.crypto.BadPaddingException;
import javax.crypto.IllegalBlockSizeException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.helpers.DefaultHandler;

import com.wechat.common.parameter.ProjectConstant;
import com.wechat.common.util.encrypt.MD5Coding;
import com.wechat.common.util.http.HTTPUtils;
import com.wechat.common.util.serial.WechatpaySerial;

import javax.servlet.http.HttpServletResponse;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;

public class WeChatUtil {
	private static Logger log = LoggerFactory.getLogger(WeChatUtil.class);
	public static void main(String[] args) {
		X509Certificate certificate = getX509Certificate(ProjectConstant.certificate);
		try {
			String b = WechatpaySerial.rsaEncryptOAEP("aa", certificate);
			try {
				System.out.println("aaaaa:"+WechatpaySerial.rsaDecryptOAEP(b, WeChatUtil.getPrivateKey(ProjectConstant.yourPrivateKey)));
			} catch (BadPaddingException e) {
				e.printStackTrace();
			}
		} catch (IllegalBlockSizeException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}


	}

	/**
	 * 根据字符串获取证书X509Certificate
	 * @param str
	 * @return
	 */
	public static X509Certificate getX509Certificate(String str){
		InputStream is = null;
		BufferedInputStream bis = null;
		try {
			is = new ByteArrayInputStream(str.getBytes("UTF-8"));
			bis = new BufferedInputStream(is);
			CertificateFactory cf = CertificateFactory.getInstance("X509");
		    X509Certificate cert = (X509Certificate) cf.generateCertificate(bis);
		    cert.checkValidity();
		    return cert;
		} catch (Exception e) {
			e.printStackTrace();
		}finally{
			try {
				if( null != bis ){
					bis.close();
				}
				if( null != is){
					is.close();
				}
			} catch (Exception e2) {
				e2.printStackTrace();
			}
		}
		return null;
	}


	/**
	 * 解码PrivateKey
	 * @param key
	 * @return
	 */
	public static PrivateKey  getPrivateKey(String key) {
	    try {
	        byte[] byteKey = Base64.getDecoder().decode(key);
	        PKCS8EncodedKeySpec x509EncodedKeySpec = new PKCS8EncodedKeySpec(byteKey);
	        KeyFactory keyFactory = KeyFactory.getInstance("RSA");
	        PrivateKey pa = keyFactory.generatePrivate(x509EncodedKeySpec);
	        System.out.println(pa.toString());
	        return pa;
	    } catch (Exception e) {
	        e.printStackTrace();
	    }
	    return null;
	}

	/**
	   * 获取私钥。
	   *
	   * @param filename 私钥文件路径  (required)
	   * @return 私钥对象
	   */
	public static PrivateKey getPrivateKeyFile() throws IOException {

	  String content = new String(Files.readAllBytes(Paths.get("D:/资料/kangtijia/apiclient_key.pem")), "utf-8");
	  try {
	    String privateKey = content.replace("-----BEGIN PRIVATE KEY-----", "")
	        .replace("-----END PRIVATE KEY-----", "")
	        .replaceAll("\\s+", "");

	    KeyFactory kf = KeyFactory.getInstance("RSA");
	    return kf.generatePrivate(
	        new PKCS8EncodedKeySpec(Base64.getDecoder().decode(privateKey)));
	  } catch (NoSuchAlgorithmException e) {
	    throw new RuntimeException("当前Java环境不支持RSA", e);
	  } catch (InvalidKeySpecException e) {
	    throw new RuntimeException("无效的密钥格式");
	  }
	}
	public static String sha256_HMAC(Map<String,Object> params, String mch_key){
		if(params == null)
			return "";
		//去掉空值和sign参数,使用Iterator遍历可在遍历同时删除元素
		Iterator<Entry<String, Object>> it = params.entrySet().iterator();
		while(it.hasNext()){
			Entry<String, Object> en = it.next();
			String key = en.getKey();
			Object value = en.getValue();
			if(null == value || StringUtil.isEmpty(String.valueOf(value)) || key.equalsIgnoreCase("sign")){
				it.remove();
			}
		}
		//按key排序后以[key=value&key=value]形式拼接,最后拼接微信秘钥,生成sign
		List<String> paramsKeys = new ArrayList<String>(params.keySet());
		Collections.sort(paramsKeys);
		StringBuilder sb = new StringBuilder();
		for (String param_key : paramsKeys) {
			String param_value = String.valueOf(params.get(param_key));
			sb.append(param_key).append("=").append(param_value).append("&");
		}
		sb.append("key").append("=").append(mch_key);
		return HMACSHA256.sha256_HMAC(sb.toString(),mch_key);
	}

	public static String createSign(Map<String,Object> params, String mch_key){
		if(params == null){
			return "";
		}
		//去掉空值和sign参数,使用Iterator遍历可在遍历同时删除元素
		Iterator<Entry<String, Object>> it = params.entrySet().iterator();
		while(it.hasNext()){
			Entry<String, Object> en = it.next();
			String key = en.getKey();
			Object value = en.getValue();
			if(null == value || StringUtil.isEmpty(String.valueOf(value)) || key.equalsIgnoreCase("sign")){
				it.remove();
			}
		}
		//按key排序后以[key=value&key=value]形式拼接,最后拼接微信秘钥,生成sign
		List<String> paramsKeys = new ArrayList<String>(params.keySet());
		Collections.sort(paramsKeys);
		StringBuilder sb = new StringBuilder();
		for (String param_key : paramsKeys) {
			String param_value = String.valueOf(params.get(param_key));
			sb.append(param_key).append("=").append(param_value).append("&");
		}
		sb.append("key").append("=").append(mch_key);
		//生成SIGN签名
		try{
			String signValue = MD5Coding.encode2HexStr(sb.toString().getBytes("UTF-8"));
			return signValue;
		}catch (UnsupportedEncodingException e){
			e.printStackTrace();
			return "";
		}
	}
	public static String mapFormatXML(Map<String,Object> params){
		//按KEY排序
		List<String> keys = new ArrayList<String>(params.keySet());
		Collections.sort(keys);

		StringBuilder xmlBuf = new StringBuilder();
		xmlBuf.append("<xml>\r\n");

		for (String key : keys) {
			if (key.equalsIgnoreCase("sign")){
				continue;		//将sign放到最后一行
			}
			String value = String.valueOf(params.get(key));
			String cdata_Value = "<![CDATA["+value+"]]>";   //加上CDATA可转义特殊字符
			xmlBuf.append("\t<"+key+">" + cdata_Value + "</"+key+">\r\n");
		}
		xmlBuf.append("\t<sign>" + params.get("sign") + "</sign>\r\n");
		xmlBuf.append("</xml>");

		return xmlBuf.toString();
	}

	public static Map<String, Object> parseXML(String xmlString) {
        if (xmlString == null || xmlString.equals(""))
            throw (new IllegalArgumentException("argument xmlString must not be null."));
        try {
            SAXParser parser = SAXParserFactory.newInstance().newSAXParser();
            InputStream source = new ByteArrayInputStream(xmlString.getBytes("UTF-8"));
            PropertyCollector pc = new PropertyCollector();
            parser.parse(source, pc);
            return (pc.returnResult());
        } catch (Exception ex) {
            throw (new RuntimeException(ex.getMessage(), ex));
        }
    }
	private static class PropertyCollector extends DefaultHandler
    {
        private LinkedList<String> keyStack = new LinkedList<String>();

        private LinkedList<StringBuilder> valueStack = new LinkedList<StringBuilder>();

        private HashMap<String, Object> result = new HashMap<String, Object>();

        @Override
        public void startElement(String uri, String localName, String qName, Attributes attributes) {
            this.keyStack.add(qName);
            this.valueStack.add(new StringBuilder());
            return;
        }

        @Override
        public void endElement(String uri, String localName, String qName) {
            String key = this.keyStack.removeLast();
            StringBuilder value = this.valueStack.removeLast();
            result.put(key, value.toString());
            if (valueStack.size() > 0)
                this.valueStack.getLast().append('<').append(key).append('>')
                .append(value).append("</").append(key).append('>');
            return;
        }

        @Override
        public void characters(char[] ch, int start, int length) {
            this.valueStack.getLast().append(ch, start, length);
            return;
        }

        public Map<String, Object> returnResult() {
            this.result.remove("xml"); // 解析类的一个bug.这里随便修正下，后期再改
            return (this.result);
        }
    }
	public static String postWX(String wxapi, String postXmlString){
log.info("请求微信前参数：postWX:{},参数：{}",wxapi,postXmlString);
		String data = "";
		try {
			data = HTTPUtils.post(wxapi,null,postXmlString);
log.info("微信返回结果：data:{}",data);
//System.out.println("微信返回信息："+data);
		} catch (Exception e) {
			e.printStackTrace();
			data = e.getMessage();
		}
		//Map<String,Object> map = parseXML(data);
		return data;
	}

	/**
     * 返回处理结果给微信, 如:Util.write2wx(response, "SUCCESS", "OK")
     * @param response
     * @param return_code
     * @param return_msg
     */
    public static void write2wx(HttpServletResponse response, String return_code, String return_msg){
        StringBuilder xmlBuf = new StringBuilder();
        xmlBuf.append("<xml>\r\n");
        xmlBuf.append("\t<return_code>" + "<![CDATA["+return_code+"]]>" + "</return_code>\r\n");
        xmlBuf.append("\t<return_msg>" + "<![CDATA["+return_msg+"]]>" + "</return_msg>\r\n");
        xmlBuf.append("</xml>");
        String xml = xmlBuf.toString();
        try {
            PrintWriter out = response.getWriter();
            out.write(xml);
            out.flush();
            out.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}
