package com.wechat.common.util;

import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

/**
 * 微信XML结果解析器
 */
public class WxXmlParser
{
    private static class PropertyCollector extends DefaultHandler
    {
        private LinkedList<String> keyStack = new LinkedList<String>();

        private LinkedList<StringBuilder> valueStack = new LinkedList<StringBuilder>();

        private HashMap<String, Object> result = new HashMap<String, Object>();

        @Override
        public void startElement(String uri, String localName, String qName, Attributes attributes) {
            this.keyStack.add(qName);
            this.valueStack.add(new StringBuilder());
            return;
        }

        @Override
        public void endElement(String uri, String localName, String qName) {
            String key = this.keyStack.removeLast();
            StringBuilder value = this.valueStack.removeLast();
            result.put(key, value.toString());
            if (valueStack.size() > 0)
                this.valueStack.getLast().append('<').append(key).append('>')
                .append(value).append("</").append(key).append('>');
            return;
        }

        @Override
        public void characters(char[] ch, int start, int length) {
            this.valueStack.getLast().append(ch, start, length);
            return;
        }

        public Map<String, Object> returnResult() {
            this.result.remove("xml"); // 解析类的一个bug.这里随便修正下，后期再改
            return (this.result);
        }
    }

    public static Map<String, Object> parseXML(String xmlString) {
        if (xmlString == null || xmlString.equals(""))
            throw (new IllegalArgumentException("argument xmlString must not be null."));
        try {
            SAXParser parser = SAXParserFactory.newInstance().newSAXParser();
            InputStream source = new ByteArrayInputStream(xmlString.getBytes("UTF-8"));
            PropertyCollector pc = new PropertyCollector();
            parser.parse(source, pc);
            return (pc.returnResult());
        } catch (Exception ex) {
            throw (new RuntimeException(ex.getMessage(), ex));
        }
    }

    public static Map<String, Object> parseXML(InputStream xmlStream) {
        if (xmlStream == null)
            throw (new IllegalArgumentException("argument xmlStream must not be null."));
        try {
            SAXParser parser = SAXParserFactory.newInstance().newSAXParser();
            PropertyCollector pc = new PropertyCollector();
            parser.parse(xmlStream, pc);
            return (pc.returnResult());
        } catch (Exception ex) {
            throw (new RuntimeException(ex.getMessage(), ex));
        }
    }
}
