package com.wechat.common.util.encrypt;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.concurrent.ConcurrentHashMap;

public class MD5Coding {
	private static int INT_START = 2;
	private static int INT_END = 7;
	private static String LIKE_NAME = "comtika2018_actor";
	private static String MD5_DIGEST = "md5";
	private static ConcurrentHashMap<String, MessageDigest> digestMap = new ConcurrentHashMap<String, MessageDigest>();
	private static StringBuffer sb = new StringBuffer();

	/**
	 * <b>MD5算法加密字节数组</b><br>
	 * 
	 * @param bytes
	 * 
	 * @return 加密后的字节数组,若加密失败返回null
	 */
	public static byte[] encode(byte[] bytes) {
		try {
			MessageDigest digest = digestMap.get(MD5_DIGEST);
			if(digest == null){
				digest = MessageDigest.getInstance("MD5");
			}
			digest.update(bytes);
			byte[] digesta = digest.digest();
			return digesta;
		} catch (Exception e) {
			return null;
		}
	}
	/**
	 * 加强MD5加密
	 * 谁都不知道结束
	 */
	public static String encode(String plaintext){
		String lv1 = encode2HexStr(encode(plaintext.getBytes()));
		sb.setLength(0);
		sb.append(lv1.substring(INT_START, INT_END));
		sb.append(LIKE_NAME);
		sb.append(lv1);
		return encode2HexStr(encode(sb.toString().getBytes()));
	}
	/**
	 * 加密没有加盐 需要用户加盐
	 */
	public static String encodeNoSalt(String plaintext){
		return encode2HexStr(encode(plaintext.getBytes()));
	}
	/**
	 * 加密没有加盐 需要用户加盐
	 */
	public static String encodeAddSaltCustom(String plaintext,String salt){
		if(salt == null){
			return encodeNoSalt(plaintext);
		}
		return encode2HexStr(encode((plaintext+salt).getBytes()));
	}

	/**
	 * 用MD5算法加密后再转换成Hex String
	 * 
	 * @param bytes
	 * 
	 * @return String
	 */
	public static String encode2HexStr(byte[] bytes) {
		return HexUtil.bytes2HexStr(encode(bytes));
	}

	/**
	 * 用MD5算法加密后再转换成BASE64 String
	 * 
	 * @param bytes
	 * 
	 * @return String
	 */
	public static String encode2Base64(byte[] bytes) {
		return BASE64Coding.encode(encode(bytes));
	}

	/**
	 * 计算文件的md5
	 * 
	 * @param filePath
	 *            文件路径
	 * 
	 * @return md5结果,若加密失败,则返回null
	 */
	public static byte[] encodeFile(String filePath) {
		try {
			MessageDigest digest = digestMap.get(MD5_DIGEST);
			if(digest == null){
				digest = MessageDigest.getInstance("MD5");
			}
			FileInputStream fis = new FileInputStream(filePath);
			byte[] buffer = new byte[1024];
			byte[] digesta = null;
			int readed = -1;
			try {
				while ((readed = fis.read(buffer)) != -1) {
					digest.update(buffer, 0, readed);
				}
				digesta = digest.digest();
			} catch (IOException e) {
				e.printStackTrace();
			} finally {
				try {
					fis.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			return digesta;
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return null;
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * 计算文件的md5,转换成Hex String
	 * 
	 * @param filePath
	 *            文件路径
	 * 
	 * @return md5结果，若加密失败，则返回null
	 */
	public static String encodeFile2HexStr(String filePath) {
		return HexUtil.bytes2HexStr(encodeFile(filePath));
	}

	/**
	 * 计算文件的md5,转换成Base64 String
	 * 
	 * @param filePath
	 *            文件路径
	 * 
	 * @return md5结果，若加密失败，则返回null
	 */
	public static String encodeFile2Base64(String filePath) {
		byte[] bytes = encodeFile(filePath);
		if (bytes == null) {
			return null;
		}
		return BASE64Coding.encode(bytes);
	}
	
}
