package com.wechat.common.util.serial;

import java.io.IOException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.util.Base64;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import java.security.cert.X509Certificate;

public class WechatpaySerial {
	
	/**
	 * RSA加密
	 * @param message
	 * @param certificate
	 * @return
	 * @throws IllegalBlockSizeException
	 * @throws IOException
	 */
	public static String rsaEncryptOAEP(String message, X509Certificate certificate)
		    throws IllegalBlockSizeException, IOException {
		  try {
			  Cipher cipher = Cipher.getInstance("RSA/ECB/OAEPWithSHA-1AndMGF1Padding");
			  cipher.init(Cipher.ENCRYPT_MODE, certificate.getPublicKey());
			  byte[] data = message.getBytes("utf-8");
			  byte[] cipherdata = cipher.doFinal(data);
			  return Base64.getEncoder().encodeToString(cipherdata);
		  } catch (NoSuchAlgorithmException | NoSuchPaddingException e) {
			  throw new RuntimeException("当前Java环境不支持RSA v1.5/OAEP", e);
		  } catch (InvalidKeyException e) {
			  throw new IllegalArgumentException("无效的证书", e);
		  } catch (IllegalBlockSizeException | BadPaddingException e) {
			  throw new IllegalBlockSizeException("加密原串的长度不能超过214字节");
		  }
	}
	
	/**
	 * 解密示例
	 * @param ciphertext
	 * @param privateKey
	 * @return
	 * @throws BadPaddingException
	 * @throws IOException
	 */
	 public static String rsaDecryptOAEP(String ciphertext, PrivateKey privateKey)
		      throws BadPaddingException, IOException {
		    try {
		    	Cipher cipher = Cipher.getInstance("RSA/ECB/OAEPWithSHA-1AndMGF1Padding");
		    	cipher.init(Cipher.DECRYPT_MODE, privateKey);
		    	byte[] data = Base64.getDecoder().decode(ciphertext);
		    	return new String(cipher.doFinal(data), "utf-8");
		    } catch (NoSuchPaddingException | NoSuchAlgorithmException e) {
		    	throw new RuntimeException("当前Java环境不支持RSA v1.5/OAEP", e);
		    } catch (InvalidKeyException e) {
		    	throw new IllegalArgumentException("无效的私钥", e);
		    } catch (BadPaddingException | IllegalBlockSizeException e) {
		    	throw new BadPaddingException("解密失败");
		    }
		  }
	
}
