package com.wechat.controller;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.wechat.common.action.BaseController;
import com.wechat.common.entity.participation.RefundParticipation;
import com.wechat.common.entity.pay.jsapi.JsapiUnifiedOrder;
import com.wechat.common.response.RespWrapper;
import com.wechat.service.pay.WeChatPayJsapiService;

@RestController
@RequestMapping("/wxpay/jsapi")
public class WeChatPayJsapiController extends BaseController {

	@Autowired
	private WeChatPayJsapiService weChatPayJsapiService;

	/**
	 * 微信小程序统一下单接口
	 * 
	 * @return
	 */
	@PostMapping("/unifiedOrder")
	public RespWrapper<Object> jsapiUnifiedOrder(@RequestBody JsapiUnifiedOrder unifiedOrder) {
		return weChatPayJsapiService.jsapiUnifiedOrder(unifiedOrder);
	}
	
	/**
	 * 微信异步回调接口
	 * @param request
	 * @param response
	 */
	@PostMapping(value = "/notify")
	public void notifyOrder(HttpServletRequest request,HttpServletResponse response) {
		weChatPayJsapiService.notifyOrder(request,response);
	}
	
	/**
	 * 根据微信支付单号发起微信退款
	 * @param request
	 * @param response
	 */
	@PostMapping(value = "/refund")
	public RespWrapper<Object> refundByOutOrderNo(@RequestBody RefundParticipation params) {
		return weChatPayJsapiService.refundByOutOrderNo(params);
	}
	
	/**
	 * 根据退款单号查询微信退款  -- 外部调用
	 * @param request
	 * @param response
	 */
	@GetMapping(value = "/queryRefundByrefundId")
	public RespWrapper<Object> queryRefundByOutOrderNo(String refundId) {
		return weChatPayJsapiService.queryRefundByrefundId(refundId);
	}
	
	/**
	 * 根据微信退款单号查询微信退款状态
	 * @param refundId 微信退款单号
	 * @return
	 */
	@PostMapping(value = "/queryWechatRefundStatus")
	public RespWrapper<Object> queryWechatRefundStatus(String refundId) {
		return weChatPayJsapiService.queryWechatRefundStatus(refundId);
	}
	
	/**
	 * 微信退款通知接口
	 * @param request
	 * @param response
	 */
	@PostMapping(value = "/notifyRefund")
	public void notifyRefund(HttpServletRequest request,HttpServletResponse response) {
		weChatPayJsapiService.notifyRefund(request,response);
	}
	
	/**
	 * 查询分账账单
	 * @param subMchid
	 * @param BillDate  日期格式为2020-05-01
	 * @return
	 */
	@GetMapping(value = "/querySubLedgerBill")
	public RespWrapper<Object> querySubLedgerBill(String subMchid ,String billDate){
		return weChatPayJsapiService.querySubLedgerBill(subMchid,billDate);
	}
	
	
}
