package com.wechat.controller.sepAccount;


import com.wechat.common.action.BaseController;
import com.wechat.common.entity.sepAccount.KtjSepUser;
import com.wechat.common.entity.sepAccount.help.SepReceiverHelp;
import com.wechat.common.entity.sepAccount.help.SepUserHandleHelp;
import com.wechat.common.entity.sepAccount.help.UserSepReceiver;
import com.wechat.common.response.RespWrapper;
import com.wechat.common.util.StringUtil;
import com.wechat.service.sepAccount.ISepAccountService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Api(description = "分账对外接口")
@RestController
@RequestMapping("sepAccount")
public class SepAccountController extends BaseController {


    @Autowired
    private ISepAccountService iSepAccountService;


    @ApiOperation(value = "新增分账人接口")
    @PostMapping("addSepReceive")
    public RespWrapper<Boolean> addSepReceive(@RequestBody UserSepReceiver userSepReceiver){
        Assert.notNull(userSepReceiver.getSubMchId(),"服务商子商户不能为空");
        List<SepReceiverHelp> sepUserList = userSepReceiver.getSepUserList();
        if(sepUserList == null || sepUserList.isEmpty())
            return RespWrapper.makeFailResp("分账人不能为空");
        int num = 0;
        for(SepReceiverHelp help:sepUserList){
            if(StringUtil.isEmpty(help.getNickName()))
                return RespWrapper.makeFailResp("分账人不能为空");
            if(StringUtil.isEmpty(help.getWxAccount()))
                return RespWrapper.makeFailResp("分账微信账号不能为空");
            if(StringUtil.isEmpty(help.getSepRate()))
                return RespWrapper.makeFailResp("分账费率不能为空");
            Float fl = Float.valueOf(help.getSepRate());
            num += (int) (fl.floatValue()*100);
        }
        if(num > 30) return RespWrapper.makeFailResp("分账订单金额不能超过30%");
        return iSepAccountService.addSepReceive(userSepReceiver);
    }
    @ApiOperation(value = "查询自己分账接口")
    @PostMapping("findSepUserList")
    public RespWrapper<List<KtjSepUser>> findSepUserList(@RequestBody SepUserHandleHelp help){
        if(StringUtil.isEmpty(help.getSubMchId()))
            return RespWrapper.makeFailResp("特约商户号不能为空");
        return iSepAccountService.findSepUserList(help);
    }
    @ApiOperation(value = "分账操作")
    @PostMapping("sepAccountHandle")
    public RespWrapper<String> sepAccountHandle(@RequestBody SepUserHandleHelp handleHelp){
        if(StringUtil.isEmpty(handleHelp.getSubMchId()))
            return RespWrapper.makeFailResp("特约商户号不为空");
        if(StringUtil.isEmpty(handleHelp.getTransactionId()))
            return RespWrapper.makeFailResp("微信支付订单号不能为空");
        return iSepAccountService.sepAccountHandle(handleHelp);
    }

    /**
     *         FINISHED 分账成功
     *    case "ACCEPTED":
     *        return RespWrapper.makeFailResp("受理成功");
     *    case "PROCESSING":
     *        return RespWrapper.makeFailResp("处理中");
     *    case "FINISHED":
     *        return RespWrapper.makeSuccessResp(true);
     *    case "CLOSED":
     *        return RespWrapper.makeFailResp("处理失败，已关单");
     * @param outOrderNo
     * @return
     */
    @ApiOperation(value = "根据outOrderNo编号查询分账结果")
    @GetMapping("findSepAccountResult/{outOrderNo}")
    public RespWrapper<String> findSepAccountResult(@PathVariable("outOrderNo")String outOrderNo){
        return iSepAccountService.findSepAccountResult(outOrderNo);
    }
    @ApiOperation(value = "完结分账")
    @PostMapping("noSepAccountHandle")
    public RespWrapper<Boolean> noSepAccountHandle(@RequestBody SepUserHandleHelp handleHelp){
        if(StringUtil.isEmpty(handleHelp.getSubMchId()))
            return RespWrapper.makeFailResp("特约商户号不为空");
        if(StringUtil.isEmpty(handleHelp.getTransactionId()))
            return RespWrapper.makeFailResp("微信支付订单号不能为空");
        return iSepAccountService.noSepAccountHandle(handleHelp);
    }


}
