package com.wechat.service.merchant.impl;

import java.security.cert.X509Certificate;
import java.util.HashMap;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.wechat.common.entity.merchant.KtjMerchantDetails;
import com.wechat.common.entity.merchant.KtjMiniProgramDetails;
import com.wechat.common.entity.merchant.KtjRegisterMessage;
import com.wechat.common.entity.merchant.WeChatMerchant;
import com.wechat.common.entity.merchant.RegisterMerchant.RegisterMerchant;
import com.wechat.common.entity.merchant.bank_account_info.BankAccountInfo;
import com.wechat.common.entity.merchant.business_info.BusinessInfo;
import com.wechat.common.entity.merchant.contact_info.ContactInfo;
import com.wechat.common.entity.merchant.settlement_info.SettlementInfo;
import com.wechat.common.entity.merchant.subject_info.SubjectInfo;
import com.wechat.common.entity.register.RegisterData;
import com.wechat.common.entity.register.KtjRegisterData;
import com.wechat.common.enums.RegisterStatus;
import com.wechat.common.response.RespCode;
import com.wechat.common.response.RespWrapper;
import com.wechat.common.util.ComtikaWeChatUtil;
import com.wechat.common.util.StringUtil;
import com.wechat.common.util.WeChatHttp;
import com.wechat.common.util.WeChatUtil;
import com.wechat.common.util.http.HTTPUtils;
import com.wechat.dao.SysRegisterDataDao;
import com.wechat.dao.merchant.MerchantDetailsDao;
import com.wechat.dao.merchant.MiniProgramDetailsDao;
import com.wechat.dao.merchant.RegisterMessageDao;
import com.wechat.service.merchant.MerchantService;

@Service
public class MerchantServiceImpl implements MerchantService {

	@Autowired
	private MerchantDetailsDao merchantDetailDao;

	@Autowired
	private RegisterMessageDao registerMessageDao;

	@Autowired
	private MiniProgramDetailsDao miniProgramDetailsDao;
	
	@Autowired
	private SysRegisterDataDao sysRegisterDataDao;
	
	@Override
	public RespWrapper<Object> weChatRegisterMerchant(
			RegisterMerchant registerMerchant) {
		Map<String, String> map = new HashMap<String, String>();
		// 生成微信申请单号
		String businessCode = ComtikaWeChatUtil.getBusinessCode();
		try {
			// 参数验证
			RespWrapper<Object> flag = registerMerchant.parameterValidation();
			if (RespCode.SUCCESS != flag.getCode()) {
				return flag;
			}
			ComtikaWeChatUtil.PubKey pubKey = ComtikaWeChatUtil.findPubKey();
			X509Certificate certificate = WeChatUtil
					.getX509Certificate(pubKey.prikey);

			WeChatMerchant wm = new WeChatMerchant();
			wm.setBusiness_code(businessCode);
			// 超级管理员信息
			ContactInfo contactInfo = new ContactInfo();
			contactInfo.setInfo(registerMerchant, certificate, wm);

			// 主体资料
			SubjectInfo subjectInfo = new SubjectInfo();
			subjectInfo.setInfo(registerMerchant, certificate, wm);

			// 经营资料 -- start
			BusinessInfo businessInfo = new BusinessInfo();
			businessInfo.setInfo(registerMerchant, wm);

			// 结算规则--start
			SettlementInfo settlementInfo = new SettlementInfo();
			settlementInfo.setInfo(registerMerchant, wm);

			// 结算银行账户 - start
			BankAccountInfo bankAccountInfo = new BankAccountInfo();
			bankAccountInfo.setInfo(registerMerchant, certificate, wm);

			String body = JSONObject.toJSONString(wm);
			// 获取签名
			String auth = ComtikaWeChatUtil.findAuthorization("POST","/v3/applyment4sub/applyment/", body);

			map.put("Charsert", "UTF-8");
			map.put("connection", "Keep-Alive");
			map.put("Wechatpay-Serial", pubKey.serialNo);
			map.put("Content-Type", "application/json");
			map.put("Accept", "application/json");
			map.put("Authorization", auth);
System.out.println(body);
			String data = "";
			try {
				//data = "{\"applyment_id\":2000002138689059111}";
				data = HTTPUtils.post("https://api.mch.weixin.qq.com/v3/applyment4sub/applyment/",map,body);
			} catch (Exception e) {
				e.printStackTrace();
				data = e.getMessage();
			}

System.out.println("测试调用微信返回结果:" + data);
			JSONObject jsonObject = JSONObject.parseObject(data);
			// 保存请求信息
			KtjRegisterMessage registerMessage = new KtjRegisterMessage();
			registerMessage.setBody(body);
			registerMessage.setBusinessCode(businessCode);
			registerMessage.setData(data);
			registerMessageDao.insertTemplate(registerMessage, true);
			String applymentId = String.valueOf(jsonObject.get("applyment_id"));
			if (StringUtil.isEmpty(applymentId)) {
				return RespWrapper.makeFailResp("商户进件出错，请重试！");
			}
			KtjMerchantDetails merchantDetails = new KtjMerchantDetails();
			merchantDetails.updateInfo(registerMerchant);
			merchantDetails.setApplymentId(applymentId);
			merchantDetails.setBusinessCode(businessCode);
			merchantDetailDao.insertTemplate(merchantDetails, true);

			KtjMiniProgramDetails miniProgramDetails = new KtjMiniProgramDetails();
			miniProgramDetails.uodateInfo(registerMerchant,merchantDetails.getId());
			miniProgramDetailsDao.insert(miniProgramDetails);
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeErrorResp("服务器错误,请重试！");
		}
		map.clear();
		map.put("businessCode", businessCode);
		return RespWrapper.makeSuccessResp(map);
	}


	@Override
	public RespWrapper<Object> queryWeChatRegister(String businessCode) {
		try {
			if(StringUtil.isEmpty(businessCode)){
				return RespWrapper.makeErrorResp("业务申请编号不能为空！");
			}
			Map<String,Object> map = new HashMap<String, Object>();
			map.put("businessCode", businessCode);
			map =  merchantDetailDao.queryWeChatRegister(map);
			//Integer registerStatus = Integer.parseInt(""+map.get("registerStatus"));
			map.put("businessCode", businessCode);
			return RespWrapper.makeSuccessResp(map);
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeErrorResp("服务器错误！");
		}
	}

	

	@Override
	public RespWrapper<Object> queryWeChatMerchantRegisterbyBusinessCode(String businessCode) {
		try {
			if(StringUtil.isEmpty(businessCode)){
				return RespWrapper.makeErrorResp("业务申请编号不能为空！");
			}
			//判断业务申请编号是否存在
			KtjMerchantDetails merchantDetails = new KtjMerchantDetails();
			merchantDetails.setBusinessCode(businessCode);
			merchantDetails = merchantDetailDao.templateOne(merchantDetails);
			if( null == merchantDetails){
				return RespWrapper.makeErrorResp("业务申请编号不存在！");
			}
			//请求微信
			String url = "https://api.mch.weixin.qq.com/v3/applyment4sub/applyment/business_code/" + businessCode ;
			String auth = ComtikaWeChatUtil.findAuthorization("GET", "/v3/applyment4sub/applyment/business_code/"+businessCode, "");
			Map<String ,String > map = new HashMap<String ,String>();
			map.put("Authorization", auth);
			String data = WeChatHttp.weChatGet(url,map);
			RegisterData registerData = JSONObject.parseObject(data, RegisterData.class);
			KtjRegisterData sysRegisterData = new KtjRegisterData();
			//json解析赋值
			sysRegisterData.upInfo(registerData);
			KtjMerchantDetails merchant = new KtjMerchantDetails();
			Integer registerStatus = RegisterStatus.getIdByType(sysRegisterData.getApplymentState());
			merchant.setId(merchantDetails.getId());
			if(registerStatus == 2 || registerStatus == 7 ){//审核失败
				registerStatus = 3;
			}else if(registerStatus == 6){ //审核完成
				registerStatus = 2;
			}else{
				registerStatus = 1;//审核中
			}
			merchant.setRegisterStatus(registerStatus);
			merchant.setRegisterMessage(sysRegisterData.getApplymentStateMsg());
			if(!StringUtil.isEmpty(sysRegisterData.getSubMchid())){
				merchant.setSubMchid(sysRegisterData.getSubMchid());
			}
			merchantDetailDao.updateTemplateById(merchant);
			return RespWrapper.makeSuccessResp(sysRegisterData);
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeErrorResp("服务器错误！");
		}
		
	}

	public static void main(String[] args) throws Exception {
		//new MerchantServiceImpl().queryWeChatMerchantRegisterbyApplymentId("2000002138945949");
		/*String str = "{\"applyment_id\":2000002138945949,\"applyment_state\":\"APPLYMENT_STATE_AUDITING\",\"applyment_state_msg\":\"1、申请单正在审核中；2、可让超级管理员用微信打开“签约链接”，完成绑定微信号后，申请单进度将通过微信公众号通知超级管理员，引导完成后续步骤\",\"audit_detail\":[{\"field\": \"id_card_copy\",\"field_name\": \"身份证复印件\",\"reject_reason\": \"身份证背面识别失败，请上传更清晰的身份证图片。\"}],\"business_code\":\"Comtika_22A6DA6B776E4A23B93F1DED37E7ED871585995193\",\"sign_url\":\"https://pay.weixin.qq.com/index.php/extend/applyment4sub_weapp/showQRCode?applyment_id=2000002138945949&sign=cabe88800efd38708d1d93cd5af1bab9\"}";
		RegisterData registerData = JSONObject.parseObject(str, RegisterData.class);
		System.out.println(registerData.toString());
		System.out.println(registerData.getAudit_detail());
		String aa = registerData.getAudit_detail().replaceAll("\\[", "").replaceAll("]", "");
		System.out.println(aa);
		AuditDetail auditDetail = JSONObject.parseObject(aa, AuditDetail.class);
		System.out.println(auditDetail.toString());*/
		String businessCode = "Comtika_22A6DA6B776E4A23B93F1DED37E7ED871585995193";
		String url = "https://api.mch.weixin.qq.com/v3/applyment4sub/applyment/business_code/" + businessCode ;
		String auth = ComtikaWeChatUtil.findAuthorization("GET", "/v3/applyment4sub/applyment/business_code/"+businessCode, "");
		Map<String ,String > map = new HashMap<String ,String>();
		map.put("Authorization", auth);
		String data = WeChatHttp.weChatGet(url,map);
		System.out.println(data);
	}

	@Override
	public RespWrapper<Object> queryWeChatMerchantRegisterbyApplymentId(String applymentId) {
		try {
			if(StringUtil.isEmpty(applymentId)){
				return RespWrapper.makeFailResp("请求参数有误，申请单号applymentId不能为空！");
			}
			//判断申请单号是否存在
			KtjMerchantDetails merchantDetails = new KtjMerchantDetails();
			merchantDetails.setApplymentId(applymentId);
			merchantDetails = merchantDetailDao.templateOne(merchantDetails);
			if( null == merchantDetails){
				return RespWrapper.makeErrorResp("申请单号不存在！");
			}
			//请求微信获取数据
			String url = "https://api.mch.weixin.qq.com/v3/applyment4sub/applyment/applyment_id/" + applymentId;
			String auth = ComtikaWeChatUtil.findAuthorization("GET", "/v3/applyment4sub/applyment/applyment_id/"+applymentId, "");
			Map<String ,String > map = new HashMap<String ,String>();
			map.put("Authorization", auth);
			String data = WeChatHttp.weChatGet(url,map);
			//json数据解析
			RegisterData registerData = JSONObject.parseObject(data, RegisterData.class);
			KtjRegisterData sysRegisterData = new KtjRegisterData();
			//json解析赋值到数据库实体类
			sysRegisterData.upInfo(registerData);
			KtjMerchantDetails merchant = new KtjMerchantDetails();
			Integer registerStatus = RegisterStatus.getIdByType(sysRegisterData.getApplymentState());
			merchant.setId(merchantDetails.getId());
			if(registerStatus == 2 || registerStatus == 7 ){//审核失败
				registerStatus = 3;
			}else if(registerStatus == 6){ //审核完成
				registerStatus = 2;
			}else{
				registerStatus = 1;//审核中
			}
			merchant.setRegisterStatus(registerStatus);
			merchant.setRegisterMessage(sysRegisterData.getApplymentStateMsg());
			if(!StringUtil.isEmpty(sysRegisterData.getSubMchid())){
				merchant.setSubMchid(sysRegisterData.getSubMchid());
			}
			merchantDetailDao.updateTemplateById(merchant);
			//如果请求返回的结果不需要存储，可以删除以下代码
			KtjRegisterData sys = new KtjRegisterData();
			sys.setApplymentId(applymentId);
			sys = sysRegisterDataDao.templateOne(sys);
			if( null == sys){
				sysRegisterDataDao.insert(sysRegisterData);
			}else{
				sysRegisterData.setId(sys.getId());
				sysRegisterDataDao.updateTemplateById(sysRegisterData);
			}
			return RespWrapper.makeSuccessResp(sysRegisterData);
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeErrorResp("服务器错误！");
		}
	}
}
