package com.wechat.service.pay.impl;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.fastjson.JSONObject;
import com.wechat.common.util.ComtikaHttpsUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.wechat.common.entity.NotifyOrder;
import com.wechat.common.entity.SubLedgerBill;
import com.wechat.common.entity.merchant.KtjMerchantDetails;
import com.wechat.common.entity.order.KtjJsapiOrder;
import com.wechat.common.entity.order.KtjRefundOrder;
import com.wechat.common.entity.participation.RefundParticipation;
import com.wechat.common.entity.pay.jsapi.JsapiUnifiedOrder;
import com.wechat.common.entity.sepAccount.KtjReceiversUser;
import com.wechat.common.entity.sepAccount.KtjSepRecord;
import com.wechat.common.enums.RefundStatus;
import com.wechat.common.parameter.ProjectConstant;
import com.wechat.common.response.RespCode;
import com.wechat.common.response.RespWrapper;
import com.wechat.common.util.StringUtil;
import com.wechat.common.util.WeChatUtil;
import com.wechat.common.util.http.HTTPUtils;
import com.wechat.dao.merchant.MerchantDetailsDao;
import com.wechat.dao.order.KtjJsapiOrderDao;
import com.wechat.dao.order.KtjRefundOrderDao;
import com.wechat.dao.pay.WeChatPayJsapiDao;
import com.wechat.dao.sepAccount.SepRecordDao;
import com.wechat.service.pay.WeChatPayJsapiService;

@Service
public class WeChatPayJsapiServiceImpl implements WeChatPayJsapiService{

	@Autowired
	private WeChatPayJsapiDao weChatPayJsapiDao;

	@Autowired
	private KtjJsapiOrderDao ktjJsapiOrderDao;

	@Autowired
	private KtjRefundOrderDao ktjRefundOrderDao;

	@Autowired
	private SepRecordDao sepRecordDao;
	
	@Autowired
	private MerchantDetailsDao merchantDetailDao;
	
	@Override
	public RespWrapper<Object> jsapiUnifiedOrder(JsapiUnifiedOrder unifiedOrder) {
		try {
			System.out.print("入参：" + unifiedOrder.toString());
			//参数验证
			RespWrapper<Object> flag = unifiedOrder.parameterValidation();
			if (RespCode.SUCCESS != flag.getCode()) {
				return flag;
			}
			//查找商户所配置的小程序
			Map<String,Object> map = weChatPayJsapiDao.queryAppId(unifiedOrder);
			if(null == map || null == map.get("id")){
				return RespWrapper.makeFailResp("特邀商户号不存在，请检查！");
			}
			//生成微信请求数据
			Map<String,Object> data = createMap(map,unifiedOrder);
			String postXmlString = WeChatUtil.mapFormatXML(data);
			String xmlStr = WeChatUtil.postWX(ProjectConstant.jsapiUrl, postXmlString);
			//将微信返回的xml转成map集合
			Map<String, Object> httpResp = WeChatUtil.parseXML(xmlStr);
			//存储订单数据
			KtjJsapiOrder order = new KtjJsapiOrder();
			order.assignment(unifiedOrder,data.get("out_trade_no"),map.get("miniProgramSubAppid"));
			ktjJsapiOrderDao.insert(order);
			//返回微信支付数据
			return returnMap(httpResp,map);
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeErrorResp("服务器错误，请稍后重试！");
		}

	}

	/**
	 * 生成微信支付数据
	 * @param httpResp
	 * @param map
	 * @return
	 */
	private RespWrapper<Object> returnMap(Map<String, Object> httpResp,Map<String, Object> map) {
		Map<String, Object> respMap = new HashMap<>();
		respMap.put("appId", map.get("miniProgramSubAppid"));
        respMap.put("timeStamp", String.valueOf(System.currentTimeMillis() / 1000)); // 当前的时间戳,秒
        respMap.put("nonceStr", UUID.randomUUID().toString().replaceAll("-", "").toUpperCase()); // 随机字符串
        respMap.put("package", "prepay_id=" + httpResp.get("prepay_id")); // 微信统一下单接口返回的prepay_id参数值，提交格式如：prepay_id=wx112738712639781263128736
        respMap.put("signType", "MD5"); // 签名算法,暂时只支持MD5
        respMap.put("paySign", WeChatUtil.createSign(respMap, ProjectConstant.api)); // 签名
		return RespWrapper.makeSuccessResp(respMap);
	}

	private Map<String, Object> createMap(Map<String, Object> map,JsapiUnifiedOrder unifiedOrder) {
		String nonceStr =  UUID.randomUUID().toString().replaceAll("-", "").toUpperCase();
		String orderNo = "Comtika_"+ System.currentTimeMillis() / 1000 + "_"+ System.currentTimeMillis() / 1000000;
		Map<String,Object> data = new HashMap<String, Object>();
		data.put("appid", ProjectConstant.appid);
		data.put("mch_id", ProjectConstant.merchantId);
		data.put("sub_appid", map.get("miniProgramSubAppid"));
		data.put("sub_mch_id", unifiedOrder.getSubMchid());
		data.put("nonce_str", nonceStr);
		data.put("body", unifiedOrder.getBody());
		data.put("out_trade_no", orderNo);
		String profitSharing = "Y".equals(unifiedOrder.getProfitSharing()) ? "Y" : "N";
		data.put("profit_sharing",profitSharing);//设定订单是可以分账的订单
		data.put("total_fee", unifiedOrder.getTotalFee());
		data.put("spbill_create_ip", unifiedOrder.getSpbillCreateIp());
		data.put("notify_url", ProjectConstant.notifyUrl);
		data.put("sub_openid", unifiedOrder.getSubOpenid());
		data.put("trade_type", "JSAPI");
		data.put("sign", WeChatUtil.createSign(data, ProjectConstant.api));
		return data;
	}

	@SuppressWarnings("unchecked")
	@Override
	public void notifyOrder(HttpServletRequest request,HttpServletResponse response) {
		BufferedReader br = null;
		try {
			br = new BufferedReader(new InputStreamReader(request.getInputStream(), "UTF-8"));
 	        StringBuffer buffer = new StringBuffer();
 	        String line = "";
 	        while ((line = br.readLine()) != null) {
 	            buffer.append(line);
 	        }
 	        System.out.println("微信返回信息:" + buffer.toString());
 	        Map<String, Object> httpResp = WeChatUtil.parseXML(buffer.toString());
 	        String orderNo = String.valueOf(httpResp.get("out_trade_no"));//微信外部单号，就是本系统订单号
	        String transactionId = String.valueOf(httpResp.get("transaction_id"));//微信支付单号
	        //微信支付成功时间格式为yyyyMMddHHmmss，如2009年12月25日9点10分10秒表示为20091225091010
	        String timeEnd = String.valueOf(httpResp.get("time_end"));
	        SimpleDateFormat sdf =new SimpleDateFormat("yyyyMMddHHmmss" );
	        Long payTime = sdf.parse(timeEnd).getTime();
 	        KtjJsapiOrder order = new KtjJsapiOrder();
 	        order.setOrderNo(orderNo);
 	        order = ktjJsapiOrderDao.templateOne(order);
 	        if(null != order){
 	        	//通知微信
 	        	WeChatUtil.write2wx(response,"SUCCESS","OK");
 	        	//修改数据库
 	        	KtjJsapiOrder order1 = new KtjJsapiOrder();
 	        	order1.setId(order.getId());
 	        	order1.setTransactionId(transactionId);
 	        	order1.setPayTime(payTime);
 	        	order1.setOrderStatus(1);
 	        	ktjJsapiOrderDao.updateTemplateById(order1);
 	        	//回调第三方
 	        	KtjMerchantDetails merchant = new KtjMerchantDetails();
 	        	merchant.setSubMchid(order.getSubMchid());
 	        	merchant = merchantDetailDao.templateOne(merchant);
System.out.println("回调地址：" + merchant.getNotifyUrl());
 	        	if(null != merchant && !StringUtil.isEmpty(merchant.getNotifyUrl())){
 	        		int i = 3;
 	        		NotifyOrder notify = new NotifyOrder();
 	        		notify.upData(transactionId,order.getOutOrderNo(),order.getSubMchid(),payTime);
 	        		String mapJson = JSONObject.toJSONString(notify);
 	        		Map<String,String> header = headerParameters();
 	        		System.out.print("开始回调第三方，回调参数：" + mapJson);
 	        		while( i >= 1){
 	        			String str = HTTPUtils.post(merchant.getNotifyUrl(), header, mapJson);
 	        			RespWrapper<Object> data = JSONObject.parseObject(str,RespWrapper.class);
 	        			if(data.isSuccess()){
 	        				break;
 	        			}
 	        			i--;
 	        		}
 	        	}
 	        }
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	
	
	private static Map<String, String> headerParameters() {
		Map<String, String> map = new HashMap<String, String>();
		map.put("Content-Type","application/json; charset=UTF-8");
		return map;
	}


	@Override
	public RespWrapper<Object> refundByOutOrderNo(RefundParticipation params) {
		try {
			KtjJsapiOrder order = new KtjJsapiOrder();
			RespWrapper<Object> data = params.paramsInfo(order);
			if( data.getCode() != RespCode.SUCCESS){
				return data;
			}
			order = ktjJsapiOrderDao.templateOne(order);
			if( null == order){
				return RespWrapper.makeFailResp("订单号不存在，请检查！");
			}
			if(order.getOrderStatus() != 1 ){
				return RespWrapper.makeFailResp("订单未支付，请勿发起退款！");
			}
			//检查退款单号是否已存在
			KtjRefundOrder reOrder = new KtjRefundOrder();
			reOrder.setSubMchid(params.getSubMchid());
			reOrder.setOutRefundNo(params.getOutRefundNo());
			reOrder = ktjRefundOrderDao.templateOne(reOrder);
			if( null != reOrder){
				return RespWrapper.makeFailResp("退款单号已存在，请重新生成！");
			}
			//检查这笔订单是否已经发起了分账
			KtjSepRecord ktjSepRecord = new KtjSepRecord();
			ktjSepRecord.setTransactionId(params.getTransactionId());
			ktjSepRecord = sepRecordDao.templateOne(ktjSepRecord);
			if( null != ktjSepRecord){
				return RespWrapper.makeFailResp("该订单已经发起分账，请勿发起退款！");
			}
			//生成退款参数
			Map<String,Object> map = refundMap(order,params);
			String postXmlString = WeChatUtil.mapFormatXML(map);
			//使用双向证书调用微信
			String xmlStr = ComtikaHttpsUtil.doCertWX(ProjectConstant.refundUrl, postXmlString);
System.out.println("微信返回结果："+xmlStr);
			//String xmlStr = WeChatUtil.postWX(ProjectConstant.refundUrl, postXmlString);
			//将微信返回的xml转成map集合
			Map<String, Object> httpResp = WeChatUtil.parseXML(xmlStr);
			String returnCode = String.valueOf(httpResp.get("return_code"));
			String resultCode = String.valueOf(httpResp.get("result_code"));
System.out.println("returnCode:"+returnCode+",resultCode:"+resultCode);
			if("SUCCESS".equals(returnCode)){
				if("SUCCESS".equals(resultCode)){
					String refundId = String.valueOf(httpResp.get("refund_id"));
					reOrder = new KtjRefundOrder();
					reOrder.setRefundId(refundId);
					reOrder.setRefundStatus(RefundStatus.REFUND_PROCESSING.getId());
					//存储退款数据
					reOrder.paramsOrder(map,params,order.getId());
					ktjRefundOrderDao.insertTemplate(reOrder, true);
					Map<String,Object> re = new HashMap<String, Object>();
					re.put("refundId", refundId);
					re.put("respMsg", "退款请求成功！");
					return RespWrapper.makeSuccessResp(re);
				}
 			}
			return RespWrapper.makeFailResp("退款请求失败，请重新发起退款！");
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeErrorResp("服务器错误，请重新发起退款！");
		}


	}

	private Map<String, Object> refundMap(KtjJsapiOrder order,RefundParticipation params) {
		Map<String,Object> map = new HashMap<String, Object>();
		map.put("appid", ProjectConstant.appid);
		map.put("mch_id", ProjectConstant.merchantId);
		map.put("sub_appid", order.getAppid());
		map.put("sub_mch_id", order.getSubMchid());
		map.put("nonce_str", UUID.randomUUID().toString().replaceAll("-", "").toUpperCase());
		map.put("transaction_id", order.getTransactionId());
		map.put("out_refund_no", "Comtika_Refund_"+System.currentTimeMillis()+System.currentTimeMillis()/1000000);
		map.put("total_fee",order.getTotalFee());
		map.put("refund_fee", params.getRefundFee());
		map.put("refund_desc", params.getRefundDesc());
		map.put("notify_url", ProjectConstant.refundNotifyUrl);
		map.put("sign", WeChatUtil.createSign(map, ProjectConstant.api));
		return map;
	}

	@Override
	public void notifyRefund(HttpServletRequest request,HttpServletResponse response) {
		// TODO Auto-generated method stub

	}

	@Override
	public RespWrapper<Object> queryRefundByrefundId(String refundId) {
		try {
			if(StringUtil.isEmpty(refundId)){
				return RespWrapper.makeFailResp("退款订单编号不能为空！");
			}
			KtjRefundOrder reOrder = new KtjRefundOrder();
			reOrder.setRefundId(refundId);
			reOrder = ktjRefundOrderDao.templateOne(reOrder);
			if( null == reOrder){
				return RespWrapper.makeFailResp("退款订单编号不存在，请检查！");
			}
			Map<String,Object> map = new HashMap<String, Object>();
			map.put("refundStatus", reOrder.getRefundStatus());
			map.put("refundMessage", reOrder.getRefundMessage());
			map.put("refundSuccessTime", reOrder.getRefundSuccessTime());
			return RespWrapper.makeSuccessResp(map);
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeErrorResp("服务器错误，请重新查询！");
		}

	}

	@Override
	public RespWrapper<Object> queryWechatRefundStatus(String refundId) {
		try {
			if(StringUtil.isEmpty(refundId)){
				return RespWrapper.makeFailResp("微信退款单号不能为空！");
			}
			KtjRefundOrder reOrder = new KtjRefundOrder();
			reOrder.setRefundId(refundId);
			Map<String,Object> data = ktjRefundOrderDao.queryRefundOrder(reOrder);
			reOrder.setRefundStatus(Integer.parseInt(data.get("refundStatus") + ""));
			if( null == data || null == data.get("subMchid")){
				return RespWrapper.makeFailResp("微信退款单号不存在，请检查！");
			}
			if(Integer.parseInt(String.valueOf(data.get("refundStatus")))  != RefundStatus.REFUND_PROCESSING.getId()){
				return RespWrapper.makeFailResp("此退款单不是退款中的订单，请勿发起查询！");
			}
			//上面sql已经查询出数据库id，需要赋值进去，然后下面可以修改退款订单的状态
			reOrder.setId(Long.parseLong(String.valueOf(data.get("id"))));
			Map<String,Object> map = new HashMap<String, Object>();
			//如果不是退款处理中的订单，就没有必要去查询微信
			if( reOrder.getRefundStatus() != RefundStatus.REFUND_PROCESSING.getId() ){
				map.put("refundStatus", reOrder.getRefundStatus());
				map.put("refundMessage", reOrder.getRefundMessage());
				return RespWrapper.makeSuccessResp(map);
			}
			//查询微信
			map =  queryRefundMap(data);
			String postXmlString = WeChatUtil.mapFormatXML(map);
			String xmlStr = WeChatUtil.postWX(ProjectConstant.refundquery, postXmlString);
			//将微信返回的xml转成map集合
			Map<String, Object> httpResp = WeChatUtil.parseXML(xmlStr);
			Integer refundStatus;
			String status = "";
			Long refundSuccessTime = 0L;
			for (Entry<String, Object> entry : httpResp.entrySet()){
				String key = entry.getKey();
				//由于微信提供退款查询，但是退款状态的字段是不固定的refund_status_$n，$n为下标，从0开始编号。
				//所以只需要找key包含refund_status的就为微信退款状态
				if(key.contains("refund_status")){
					status = String.valueOf(entry.getValue());
					refundStatus = RefundStatus.getIdByType(status);
					reOrder.setRefundStatus(refundStatus);
				}
				//由于微信提供退款查询，但是退款状态的字段是不固定的refund_success_time_$n	，$n为下标，从0开始编号。
				//所以只需要找key包含refund_success_time的就为微信退款成功后的到账时间
				if(key.contains("refund_success_time")){
					String time = String.valueOf(entry.getValue());
					try {
						SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
						refundSuccessTime = sdf.parse(time).getTime();
						reOrder.setRefundSuccessTime(refundSuccessTime);
						reOrder.setRefundStatus(RefundStatus.REFUND_SUCCESS.getId());
						reOrder.setRefundMessage("退款成功！");
					} catch (Exception e) {
						e.printStackTrace();
					}

				}
			}
			ktjRefundOrderDao.updateTemplateById(reOrder);
			return RespWrapper.makeSuccessResp("");
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeErrorResp("服务器错误！");
		}
	}

	private Map<String, Object> queryRefundMap(Map<String, Object> data) {
		Map<String,Object> map = new HashMap<String, Object>();
		map.put("appid", ProjectConstant.appid);
		map.put("mch_id", ProjectConstant.merchantId);
		map.put("sub_appid", data.get("miniProgramSubAppid"));
		map.put("sub_mch_id",  data.get("subMchid"));
		map.put("nonce_str", UUID.randomUUID().toString().replaceAll("-", "").toUpperCase());
		map.put("refund_id", data.get("refundId"));
		map.put("transaction_id", "");
		map.put("out_trade_no", "");
		map.put("out_refund_no","");
		map.put("sign", WeChatUtil.createSign(map, ProjectConstant.api));
		return map;
	}

	@Override
	public RespWrapper<Object> querySubLedgerBill(String subMchid,String billDate) {
		try {
			if(StringUtil.isEmpty(subMchid))return RespWrapper.makeFailResp("特邀商户号不能为空！");
			if(StringUtil.isEmpty(billDate))return RespWrapper.makeFailResp("账单日期不能为空");
			if(!isDate(billDate))return RespWrapper.makeFailResp("日期格式不正确");
			SimpleDateFormat simple = new SimpleDateFormat("yyyy-MM-dd");
			Long startTime = simple.parse(billDate).getTime();		//获取当天开始时间
			Long endTime = startTime + (24 * 60 * 60 * 1000) - 1;	//获取当天结束时间
			Map<String,Object> map = new HashMap<String, Object>();
			map.put("endTime", endTime);
			map.put("startTime", startTime);
			map.put("subMchid", subMchid);
			//1.获取商户的所以支付金额
			Long payMoney = ktjJsapiOrderDao.queryPayMoney(map);
			//2.获取商户当天的退款金额
			Long refundMoney = ktjRefundOrderDao.queryRefundMoney(map);
			//1.查询当天该特邀商户的分账金额
			Long amount = sepRecordDao.querySubLedgerBill(map);
			List<Map<String,Object>> list = null;
			if( 0 == amount || "0".equals(amount)){
				list = new ArrayList<Map<String,Object>>();
			}else{
				list = sepRecordDao.querySubReceiversUser(map);
			}
			SubLedgerBill slb = new SubLedgerBill();
			slb.setPayMoney(payMoney);
			slb.setRefundMoney(refundMoney);
			slb.setAmount(amount);
			slb.setBillDate(billDate);
			slb.setSubMchid(subMchid);
			slb.setList(list);
			return RespWrapper.makeSuccessResp(slb);
		} catch (Exception e) {
			e.printStackTrace();
			return RespWrapper.makeErrorResp("服务器错误！");
		}
		
	}
	/**
     * 判断日期格式和范围
     */
	public static boolean isDate(String date){
		boolean flag = false;
		if(date.length() == 10){
			SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
		    try {
		        // 设置lenient为false.
		        // 否则SimpleDateFormat会比较宽松地验证日期，比如2007/02/29会被接受，并转换成2007/03/01
		        format.setLenient(false);
		        format.parse(date);
		        flag = true;
		    } catch (ParseException e) {
		        e.printStackTrace();
		        // 如果throw java.text.ParseException或者NullPointerException，就说明格式不对
		    }
		}
		return flag;
    }
}
