package com.wechat.service.sepAccount.impl;

import com.alibaba.fastjson.JSONObject;
import com.wechat.common.entity.merchant.KtjMerchantDetails;
import com.wechat.common.entity.order.KtjJsapiOrder;
import com.wechat.common.entity.order.KtjRefundOrder;
import com.wechat.common.entity.sepAccount.KtjReceiversUser;
import com.wechat.common.entity.sepAccount.KtjSepRecord;
import com.wechat.common.entity.sepAccount.KtjSepUser;
import com.wechat.common.entity.sepAccount.KtjSepUserRecord;
import com.wechat.common.entity.sepAccount.Receivers;
import com.wechat.common.entity.sepAccount.help.SepReceiverHelp;
import com.wechat.common.entity.sepAccount.help.SepUserHandleHelp;
import com.wechat.common.entity.sepAccount.help.UserSepReceiver;
import com.wechat.common.enums.SepAccountEnums;
import com.wechat.common.parameter.ProjectConstant;
import com.wechat.common.response.RespWrapper;
import com.wechat.common.util.ComtikaHttpsUtil;
import com.wechat.common.util.StringUtil;
import com.wechat.common.util.WeChatUtil;
import com.wechat.common.util.WxXmlParser;
import com.wechat.dao.merchant.MerchantDetailsDao;
import com.wechat.dao.order.KtjJsapiOrderDao;
import com.wechat.dao.order.KtjRefundOrderDao;
import com.wechat.dao.sepAccount.KtjReceiversUserDao;
import com.wechat.dao.sepAccount.SepRecordDao;
import com.wechat.dao.sepAccount.SepUserDao;
import com.wechat.dao.sepAccount.SepUserRecordDao;
import com.wechat.service.sepAccount.ISepAccountService;



import org.apache.http.client.utils.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;



import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class SepAccountServiceImpl implements ISepAccountService, DisposableBean {
    // 不传 使用默认的分组
    private static final String DEFAULT_GROUP_NAME = "KTJ_SEP_GROUP";
    @Autowired
    private SepUserDao sepUserDao;
    @Autowired
    private KtjJsapiOrderDao ktjJsapiOrderDao;
    @Autowired
    private SepUserRecordDao sepUserRecordDao;
    @Autowired
    private SepRecordDao sepRecordDao;
    @Autowired
    private MerchantDetailsDao merchantDetailsDao;
    @Autowired
    private KtjRefundOrderDao ktjRefundOrderDao;
    @Autowired
    private KtjReceiversUserDao ktjReceiversUserDao;
    // 删除分账
    private static final String DEL_SEP_ACCOUNT_USER = "{\"type\":\"%s\",\"account\":\"%s\"}";
    // 添加分账
    private static final String ADD_WX_ACCOUNT_USER = "{\"type\":\"%s\",\"account\":\"%s\",\"name\":\"%s\",\"relation_type\":\"%s\"}";
    private static Logger logger = LoggerFactory.getLogger(SepAccountServiceImpl.class);

    @Override
    public RespWrapper<Boolean> addSepReceive(UserSepReceiver userSepReceiver) {
        if (userSepReceiver.getGroupName() == null)
            userSepReceiver.setGroupName(DEFAULT_GROUP_NAME);
        // 这里一套太难啦，修改为 只向微信添加访问人的数据
        return doFadAddSepReceive(userSepReceiver);
        /*// 1、校验该商户是否已经添加分账功能
        RespWrapper<Boolean> checkSepFun = checkSepFunction(userSepReceiver.getSubMchId());
        if(!checkSepFun.isSuccess()) return checkSepFun;
        // 2、查询所有已经添加的分账用户
        KtjSepUser user = new KtjSepUser();
        user.setSubMchId(userSepReceiver.getSubMchId());
        List<KtjSepUser> template = sepUserDao.template(user);
        String groupName = userSepReceiver.getGroupName();
        if(StringUtil.isEmpty(groupName)) groupName = DEFAULT_GROUP_NAME;
        // 获取当前分组的数据
        List<KtjSepUser> groupSepUserMap = findGroupSepUserMap(template,groupName);
        // 去掉当前分组的账号引用
        Map<String,KtjSepUser> accountMap = findWxAccountMap(template,groupName);
        template.clear();
        // 需要保证并且请求微信
        List<KtjSepUser> saveDBAndWx = new ArrayList<>(8);
        // 只需保证到数据库
        List<KtjSepUser> saveDB = new ArrayList<>(8);
        // 删除没有引用的微信分账人
        List<KtjSepUser> delWXUser = new ArrayList<>(0);
        // 删除同组名的分账人
        List<KtjSepUser> delDBUser = new ArrayList<>(0);
        if(groupSepUserMap.isEmpty()){
            handleSepUserData(saveDBAndWx,saveDB,accountMap,userSepReceiver,groupName);
        }else {
            *//**
             * 如果分组不为空
             * 1、新增的数据在分组中，则更新分组内对应的数据
             * 2、不在分组，并且在其他的分组中，需要新数据库
             * 3、不在分组，并且不在其他分组中，需新增数据库，并请求微信新增
             *//*
            // 需要删除的数据为
            Map<String,SepReceiverHelp> tmpGroupName = new HashMap<>();
            userSepReceiver.getSepUserList().forEach(s->{
                tmpGroupName.put(s.getWxAccount(),s);
            });
            // 其他地方没有引用、或者没有新数据引用
            delWXUser = groupSepUserMap.stream().filter(s -> {
                return !(tmpGroupName.containsKey(s.getAccount()) || accountMap.containsKey(s.getAccount()));
            }).collect(Collectors.toList());
            delDBUser = groupSepUserMap.stream().filter(s->!tmpGroupName.containsKey(s.getAccount())).collect(Collectors.toList());
            groupSepUserMap.forEach(s->{
                accountMap.put(s.getAccount(),s);
            });
            groupSepUserMap.clear();
            tmpGroupName.clear();
            handleSepUserData(saveDBAndWx,saveDB,accountMap,userSepReceiver,groupName);
        }
        // 3、处理分账逻辑
        if(!saveDB.isEmpty())
            sepUserDao.insertBatch(saveDB);
        if(!saveDBAndWx.isEmpty()){
            for(KtjSepUser ktjSepUser:saveDBAndWx){
                logger.info("需要调用微信添加接口方法:{}",ktjSepUser);
                // todo
                //RespWrapper<Map<String, Object>> ck = addReceiver(ktjSepUser, userSepReceiver.getSubMchId());
                //if(!ck.isSuccess()) return RespWrapper.makeFailResp(ck.getMsg());
            }
            sepUserDao.insertBatch(saveDBAndWx);
        }
        if(!delWXUser.isEmpty()){
            for(KtjSepUser ktjSepUser:saveDBAndWx){
                logger.info("需要调用微信删除接口方法:{}",ktjSepUser);
                // todo
                //RespWrapper<Map<String, Object>> ck = delReceiver(ktjSepUser, userSepReceiver.getSubMchId());
                //if(!ck.isSuccess()) return RespWrapper.makeFailResp(ck.getMsg());
            }
        }
        delDBUser.forEach(s->{
            sepUserDao.deleteById(s.getId());
        });
        // help gc
        saveDB.clear();
        saveDBAndWx.clear();
        delWXUser.clear();
        delDBUser.clear();
        return RespWrapper.makeSuccessResp(true);*/
    }

    private RespWrapper<Boolean> doFadAddSepReceive(UserSepReceiver userSepReceiver) {
        // 1、判断是否有分账能力
        RespWrapper<Boolean> checkSepFun = checkSepFunction(userSepReceiver.getSubMchId());
        if(!checkSepFun.isSuccess()) return checkSepFun;
        // 查询该商户下配置的所有分账人
        KtjSepUser user = new KtjSepUser();
        user.setSubMchId(userSepReceiver.getSubMchId());
        List<KtjSepUser> template = sepUserDao.template(user);
        String groupName = userSepReceiver.getGroupName();
        List<SepReceiverHelp> sepUserList = userSepReceiver.getSepUserList();
        List<KtjSepUser> remoteWx = new ArrayList<>();
        int size = sepUserList.size();
        List<KtjSepUser> addDB = new ArrayList<>(size);
        if(!template.isEmpty()){
            Map<String,SepReceiverHelp> condition = new HashMap<>(size);
            sepUserList.forEach(s->{condition.put(s.getWxAccount(),s);});
            // 这里代表已经处理好的数据
            Map<String,Boolean> handleData = new HashMap<>(size);// 这里是已经处理的数据
            // 存在同组的数据时
            List<KtjSepUser> collect = template.stream().filter(s -> groupName.equals(s.getGroupName())).collect(Collectors.toList());
            // 相同组的数据已经处理完成
            handleAlikeGroupData(collect,condition,handleData);
            if(size == handleData.size()) return RespWrapper.makeSuccessResp(true);
            // 不同组的数据
            collect = template.stream().filter(s -> !groupName.equals(s.getGroupName())).collect(Collectors.toList());
            //处理不同组的数据
            handleNoLikeGroupData(addDB,collect,condition,handleData,groupName);
            if(size != handleData.size()){
                // 这里还有些需要调用微信
                sepUserList.forEach(s->{
                    if(!handleData.containsKey(s.getWxAccount())){
                        KtjSepUser ktjSepUser = initSepUser(s, groupName, userSepReceiver.getSubMchId());
                        addDB.add(ktjSepUser);
                        remoteWx.add(ktjSepUser);
                        handleData.put(s.getWxAccount(),true);
                    }
                });
            }
        }else {
            sepUserList.forEach(s->{
                KtjSepUser ktjSepUser = initSepUser(s, groupName, userSepReceiver.getSubMchId());
                addDB.add(ktjSepUser);
                remoteWx.add(ktjSepUser);
            });
        }
        for(KtjSepUser ktjSepUser:remoteWx){
            // logger.info("需要调用微信添加接口方法:{}",ktjSepUser);
            // todo
            RespWrapper<Map<String, Object>> ck = addReceiver(ktjSepUser, userSepReceiver.getSubMchId());
            if(!ck.isSuccess()) return RespWrapper.makeFailResp(ck.getMsg());
        }
        if(!addDB.isEmpty()){
            sepUserDao.insertBatch(addDB);
        }
        addDB.clear();
        return RespWrapper.makeSuccessResp(true);

    }
    //处理不同组的数据
    private void  handleNoLikeGroupData(List<KtjSepUser> addDB,List<KtjSepUser> collect, Map<String, SepReceiverHelp> condition, Map<String, Boolean> handleData, String groupName) {
        if(collect == null) return;
        collect.forEach(s->{
            // 已经处理的数据
            if(handleData.containsKey(s.getAccount())) return;
            if(condition.containsKey(s.getAccount())){
                // 这里存在，需要复制
                SepReceiverHelp help = condition.get(s.getAccount());
                addDB.add(copySepUse(groupName,help,s));
                handleData.put(s.getAccount(),true);//已经处理数据
            }
        });
    }
    private KtjSepUser copySepUse(String groupName,SepReceiverHelp help,KtjSepUser old){
        KtjSepUser user = new KtjSepUser();
        user.setStatus(true);
        user.setSepRate(help.getSepRate());
        user.setAccount(help.getWxAccount());
        user.setGroupName(groupName);
        user.setName(help.getNickName());
        user.setRelationType(old.getRelationType());
        user.setSubMchId(old.getSubMchId());
        user.setType(old.getType());
        user.setCustomRelation(old.getCustomRelation());
        return user;
    }

    private KtjSepUser initSepUser(SepReceiverHelp hr,String groupName,String subMchId){
        KtjSepUser kt = new KtjSepUser();
        kt.setAccount(hr.getWxAccount());
        kt.setSepRate(hr.getSepRate());
        kt.setGroupName(groupName);
        kt.setName(hr.getNickName());
        kt.setType("PERSONAL_SUB_OPENID");
        kt.setSubMchId(subMchId);
        kt.setRelationType("DISTRIBUTOR");
        kt.setStatus(true);
        return kt;
    }

    // 处理相同组的数据
    private void handleAlikeGroupData(List<KtjSepUser> collect,Map<String,SepReceiverHelp> condition,Map<String,Boolean> handleData){
        if(collect == null)return;
        // 处理同组数据
        collect.forEach(s->{
            String key = s.getAccount();
            if (handleData.containsKey(key))
                return;
            // 如果是在同组，并且账号一样，判断费率是否一样
            if (condition.containsKey(key)) {
                boolean isUpdateDB = false;
                SepReceiverHelp help = condition.get(key);
                String newRate = help.getSepRate();
                if (newRate.equals(s.getSepRate())) {
                    // 费率一样，不用更新数据库
                    if(!s.getStatus()){
                        s.setStatus(true);
                        isUpdateDB = true;
                    }
                }else {// 需要更新数据库
                    s.setSepRate(newRate);
                    s.setStatus(true);
                    isUpdateDB = true;
                }
                if(isUpdateDB) sepUserDao.updateTemplateById(s);
                handleData.put(key,true);//处理的数据
            }else {
                //这里是旧的数据，判断状态是否为false 如果为true 更新数据库
                if(s.getStatus()){
                    s.setStatus(false);
                    sepUserDao.updateTemplateById(s);
                }
            }
        });
    }

    @Override
    public RespWrapper<List<KtjSepUser>> findSepUserList(SepUserHandleHelp help) {
        return RespWrapper.makeSuccessResp(doFindSepUserList(help));
    }
    private List<KtjSepUser> doFindSepUserList(SepUserHandleHelp help){
        String groupName = help.getGroupName();
        if(StringUtil.isEmpty(help.getGroupName())) groupName = DEFAULT_GROUP_NAME;
        KtjSepUser user = new KtjSepUser();
        user.setStatus(true);
        user.setSubMchId(help.getSubMchId());
        user.setGroupName(groupName);
        List<KtjSepUser> template = sepUserDao.template(user);
        return template;
    }
    private static final String SEP_DESC = "康体佳—分账";
    @Override
    public RespWrapper<String> sepAccountHandle(SepUserHandleHelp handleHelp) {
        // 1、校验该商户是否已经添加分账功能
        //RespWrapper<Boolean> checkSepFun = checkSepFunction(handleHelp.getSubMchId());
    	//if(!checkSepFun.isSuccess()) return RespWrapper.makeFailResp(checkSepFun.getMsg());
        KtjMerchantDetails ktj = new KtjMerchantDetails();
        ktj.setSubMchid(handleHelp.getSubMchId());
        ktj = merchantDetailsDao.templateOne(ktj);
        if(null == ktj || ktj.getRegisterStatus() == null) return RespWrapper.makeFailResp("该商户还没有入驻,请先入驻");
        if(ktj.getRegisterStatus() == 1) return RespWrapper.makeFailResp("该商户审核中，不能进行分账人设置");
        if(ktj.getRegisterStatus() == 3) return RespWrapper.makeFailResp("该商户入驻失败，请重新入驻");
        
        List<KtjSepUser> userList = doFindSepUserList(handleHelp);
        if(userList.isEmpty()) return RespWrapper.makeFailResp("请检查是否设置了分账人");
        // 查询是否有该笔订单
        RespWrapper<KtjJsapiOrder> data = findOrder(handleHelp);
        if(!data.isSuccess()) return RespWrapper.makeFailResp(data.getMsg());
        KtjJsapiOrder order = data.getData();
        //判断是否在退款中的订单
        KtjRefundOrder refund = new KtjRefundOrder();
        refund.setKtjJsapiOrderId(order.getId());
        refund = ktjRefundOrderDao.templateOne(refund);
        if( null != refund && refund.getRefundStatus() == 2) return RespWrapper.makeFailResp("该订单正在退款中，暂不能分账！");
        //如果该订单已经退款完成，获取退款金额。
        Long refundFee = 0L;
        if( null != refund && refund.getRefundStatus() == 1) refundFee = refund.getRefundFee();
        Long totalFee = order.getTotalFee() - refundFee;//分账金额  = 支付金额 - 退款金额 
        if(totalFee == null || totalFee < 0)
            return RespWrapper.makeFailResp("订单金额错误");
        if(handleHelp.getAmount() != null && handleHelp.getAmount() > totalFee)
            return RespWrapper.makeFailResp("分账金额不能大于支付金额");
        if(handleHelp.getAmount() != null) totalFee = handleHelp.getAmount().longValue();
        // 判断该用户已经分账啦
        KtjSepRecord record = new KtjSepRecord();
        record.setOrderId(order.getId().intValue());
        if (sepRecordDao.templateCount(record) > 0) {
            return RespWrapper.makeFailResp("该订单已经分账啦,无需在调用分账");
        }
        List<KtjSepUserRecord> addUserList = new ArrayList<>();
        StringBuilder sb = new StringBuilder("[");
        boolean isOne = true;
        long startTime = System.currentTimeMillis();
        int amount = 0;
        //存储分账用户信息
        for(KtjSepUser user : userList){
            Float aFloat = Float.valueOf(user.getSepRate());
            int tmpAmount = (int) (aFloat.floatValue()*totalFee);
            KtjSepUserRecord rd = new KtjSepUserRecord();
            rd.setNickName(user.getName());
            rd.setGroupName(user.getGroupName());
            rd.setCreateTime(startTime);
            rd.setWxAccount(user.getAccount());
            rd.setAmount(tmpAmount);
            addUserList.add(rd);
            amount += tmpAmount;
            if(tmpAmount > 0){
                String desc = String.format(SEP_RECEIVERS, user.getType(), user.getAccount(), tmpAmount, SEP_DESC);
                if(isOne){
                    isOne = false;
                }else sb.append(",");
                sb.append(desc);
            }
        }
        if(!addUserList.isEmpty()){
        	//存储商户分账信息
            KtjSepUserRecord rd = new KtjSepUserRecord();
            rd.setNickName(ktj.getMerchantName());
            rd.setGroupName(addUserList.get(0).getGroupName());
            rd.setCreateTime(startTime);
            rd.setWxAccount(ktj.getSubMchid());
            rd.setAmount(Integer.parseInt((totalFee - amount) + ""));
            addUserList.add(rd);
        }
        //查询特邀商户appid
        KtjMerchantDetails merchant = new KtjMerchantDetails();
    	merchant.setSubMchid(handleHelp.getSubMchId());
    	merchant = merchantDetailsDao.templateOne(merchant);
    	
        String outOrderNo = findSepOutOrderNo();
        record.setOutOrderNo(outOrderNo);
        record.setCreateTime(startTime);
        record.setStatus("PROCESSING");//如果发起分账请求成功，则设置成微信处理中
        record.setAmount(Integer.parseInt(totalFee + ""));//分账
        record.setSubMchId(handleHelp.getSubMchId());
        record.setTransactionId(handleHelp.getTransactionId());
        record.setSep(false);
        if(isOne){
            // 说明什么都没有分账
            record.setStatus("FINISHED");
            record.setSep(true);
            sepRecordDao.insert(record,true);
        }else {
            sb.append("]");
            RespWrapper<Map<String, Object>> ck = handleSepAccount(handleHelp.getSubMchId(), handleHelp.getTransactionId(), outOrderNo, sb.toString(),merchant.getMiniProgramSubAppid());
            if (!ck.isSuccess()) {
                return RespWrapper.makeFailResp(ck.getMsg());
            }
            String order_id = (String) ck.getData().get("order_id");
            record.setWxOrderNo(order_id);
            sepRecordDao.insert(record,true);
        }
        addUserList.forEach(s->{
            s.setSepRecordId(record.getId());
        });
        sepUserRecordDao.insertBatch(addUserList);
        return RespWrapper.makeSuccessResp(outOrderNo);
    }

    /**
     *  switch (status) {
     *              FINISHED
     *             case "ACCEPTED":
     *                 return RespWrapper.makeFailResp("受理成功");
     *             case "PROCESSING":
     *                 return RespWrapper.makeFailResp("处理中");
     *             case "FINISHED":
     *                 return RespWrapper.makeSuccessResp(true);
     *             case "CLOSED":
     *                 return RespWrapper.makeFailResp("处理失败，已关单");
     *         }
     *         return RespWrapper.makeFailResp("未知原因：" + status);
     * @param outOrderNo
     * @return
     */
    @Override
    public RespWrapper<String> findSepAccountResult(String outOrderNo) {
        KtjSepRecord record = new KtjSepRecord();
        record.setOutOrderNo(outOrderNo);
        record = sepRecordDao.templateOne(record);
        if (record == null) return RespWrapper.makeFailResp("订单编号错误");
        if("FINISHED".equals(record.getStatus()))
            return RespWrapper.makeSuccessResp("FINISHED");
        Map<String, Object> condition = new HashMap<>();
        condition.put("sub_mch_id", record.getSubMchId());//sub_mchId
        condition.put("transaction_id", record.getTransactionId());
        condition.put("out_order_no", outOrderNo);
        RespWrapper<Map<String, Object>> ck = doWxRequest(SepAccountEnums.查询分账结果, condition, false);
        if (!ck.isSuccess()) return RespWrapper.makeFailResp(ck.getMsg());
        String status = (String) ck.getData().get("status");
        if(!record.getStatus().equals(status)){
            record.setStatus(status);
            sepRecordDao.updateTemplateById(record);
        }
        return RespWrapper.makeSuccessResp(status);
    }

    @Override
    public RespWrapper<Boolean> noSepAccountHandle(SepUserHandleHelp handleHelp) {
        // 1、校验该商户是否已经添加分账功能
        RespWrapper<Boolean> checkSepFun = checkSepFunction(handleHelp.getSubMchId());
        if(!checkSepFun.isSuccess()) return RespWrapper.makeFailResp(checkSepFun.getMsg());
        // 查询是否有该笔订单
        RespWrapper<KtjJsapiOrder> data = findOrder(handleHelp);
        if(!data.isSuccess()) return RespWrapper.makeFailResp(data.getMsg());
        KtjSepRecord record = new KtjSepRecord();
        String outOrderNo = findSepOutOrderNo("NOSEP");
        record.setOutOrderNo(outOrderNo);
        record.setTransactionId(handleHelp.getTransactionId());
        record.setSep(true);
        record.setAmount(0);
        record.setCreateTime(System.currentTimeMillis());
        record.setSubMchId(handleHelp.getSubMchId());
        record.setOrderId(data.getData().getId().intValue());
        record.setStatus("FINISHED");
        Map<String,Object> condition = new HashMap<>();
        condition.put("sub_mch_id", handleHelp.getSubMchId());//sub_mchId
        condition.put("appid",ProjectConstant.appid);// appid
        condition.put("transaction_id",handleHelp.getTransactionId());
        condition.put("sign_type","HMAC-SHA256");
        condition.put("out_order_no",outOrderNo);
        condition.put("description","分账完成");
        RespWrapper<Map<String, Object>> ck = doWxRequest(SepAccountEnums.完结分账, condition, true);
        if(!ck.isSuccess()) return RespWrapper.makeFailResp(ck.getMsg());
        String order_id = (String) ck.getData().get("order_id");
        record.setWxOrderNo(order_id);
        sepRecordDao.insert(record);
        return RespWrapper.makeSuccessResp(true);
    }
    
    /**
     * 查询分账状态
     * 分账单状态：
		ACCEPTED—受理成功
		PROCESSING—处理中
		FINISHED—处理完成
		CLOSED—处理失败，已关单 
     */
    @Override
    public void doFindSepAccountResult() {
    	//查询所有受理分账成功的分账订单
        KtjSepRecord record = new KtjSepRecord();
        record.setStatus("PROCESSING");
        sepRecordDao.template(record).forEach(s->{
            Map<String, Object> condition = new HashMap<>();
            condition.put("sub_mch_id", s.getSubMchId());//sub_mchId
            condition.put("transaction_id", s.getTransactionId());
            condition.put("out_order_no", s.getOutOrderNo());
            RespWrapper<Map<String, Object>> ck = doWxRequest(SepAccountEnums.查询分账结果, condition, false);
            if (ck.isSuccess()) {// 查询成功
            	Map<String, Object> map = ck.getData();
            	String status = map.get("status").toString();//分账状态
                if ("FINISHED".equals(status) || "CLOSED".equals(status)) {//这里是分账成功或者失败
                	String receivers = map.get("receivers").toString();//分账列表
                	List<Receivers> listReceivers = JSONObject.parseArray(receivers, Receivers.class);
                	Long finishTime = null;
                	if("FINISHED".equals(status)){
                		//如果是分账成功，获取分账成功时间
                    	finishTime = getReceiversTime(listReceivers);//获得商户分账成功时间
                	}
                    //修改分账记录
                	KtjSepRecord record1 = new KtjSepRecord();
                	record1.setId(s.getId());
                	record1.setStatus(status);
                	record1.setFinishTime(finishTime);
                    sepRecordDao.updateTemplateById(record1);
                    //存储分账人记录
                    KtjReceiversUser kru = null;
                    List<KtjReceiversUser> listKtjReceiversUser = new ArrayList<KtjReceiversUser>();
                    //查询所以分账人的名称
                    for (Receivers receivers2 : listReceivers) {
                    	KtjMerchantDetails ktj = null;
                    	KtjSepUser ksu = null;
                    	String name = "";
                    	//这是商户类型
                    	if("MERCHANT_ID".equals(receivers2.getType())){
                    		ktj = new KtjMerchantDetails();
                    		ktj.setSubMchid(s.getSubMchId());
                    		ktj = merchantDetailsDao.templateOne(ktj);
                    		name = null != ktj ? ktj.getMerchantName() : "";
                    	}else{
                    		ksu = new KtjSepUser();
                    		ksu.setAccount(receivers2.getAccount());
                    		ksu.setSubMchId(s.getSubMchId());
                    		ksu = sepUserDao.templateOne(ksu);
                    		name = null != ksu ? ksu.getName() : "";
                    	}
                    	kru = new KtjReceiversUser();
                    	updateTime(receivers2,record1.getId(),kru);
                    	kru.setName(name);
                    	kru.setSubMchid(s.getSubMchId());
                    	listKtjReceiversUser.add(kru);
					}
                    if(!listKtjReceiversUser.isEmpty()){
                    	ktjReceiversUserDao.insertBatch(listKtjReceiversUser);
                    }		
                }
                
            }
        });
        
    }
    
    private void updateTime(Receivers receivers2, Integer id,KtjReceiversUser kru) {
    	kru.setFailReason(receivers2.getFail_reason());
    	kru.setKtjSepRecordId(id);
    	kru.setFinishTime(SimpleDateFormatReceiversTime(receivers2.getFinish_time()));
    	kru.setUserType(receivers2.getType());
    	kru.setReceiversResult(receivers2.getResult());
    	kru.setAmount(receivers2.getAmount());
    	kru.setUserAccount(receivers2.getAccount());
	}

	//获得商户分账成功时间
    public Long getReceiversTime(List<Receivers> listReceivers){
    	try {
			if(!listReceivers.isEmpty()){
				for (Receivers receivers : listReceivers) {
					if("MERCHANT_ID".equals(receivers.getType()) && "SUCCESS".equals(receivers.getResult())){
						String finish_time = receivers.getFinish_time();
						return SimpleDateFormatReceiversTime(finish_time);
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
    }
    
    /**
     * 时间转换
     * @param time
     * @return
     */
    public Long SimpleDateFormatReceiversTime(String time){
    	try {
    		SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
    		return sdf.parse(time).getTime();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
    }
    
    public static void main(String[] args) {
    	String xml1 = "<xml>\r\n";
    	String xml2 = "</xml>\r\n";
    	String str = "<return_code><![CDATA[SUCCESS]]></return_code>\r\n";
    	str = str + "<result_code><![CDATA[SUCCESS]]></result_code>\r\n";
    	str = str + "<mch_id><![CDATA[1503800091]]></mch_id>\r\n";
    	str = str + "<sub_mch_id><![CDATA[1596395521]]></sub_mch_id>\r\n";
    	str = str + "<nonce_str><![CDATA[5299770792eef7fc]]></nonce_str>\r\n";
    	str = str + "<sign><![CDATA[7F432BF13587E1F04BFA342BE492A8EB207AAAD0F6BA6548345E3F9D6A8F3CD6]]></sign>\r\n";
    	str = str + "<transaction_id><![CDATA[4200000532202006037115674108]]></transaction_id>\r\n";
    	str = str + "<out_order_no><![CDATA[KTJ_SEP_20200603_13893223579328335]]></out_order_no>\r\n";
    	str = str + "<order_id><![CDATA[30000001672020060300991286438]]></order_id>\r\n";
    	str = str + "<status><![CDATA[FINISHED]]></status>\r\n";
    	str = str + "<receivers><![CDATA[[{\"type\":\"MERCHANT_ID\",\"account\":\"1596395521\",\"amount\":7,\"description\":\"解冻给分账方\",\"result\":\"SUCCESS\",\"finish_time\":\"20200603111533\"},{\"type\":\"PERSONAL_SUB_OPENID\",\"account\":\"oBkzO4kUQpAVRynUL-O8xuAqyiMI\",\"amount\":3,\"description\":\"康体佳—分账\",\"result\":\"SUCCESS\",\"finish_time\":\"20200603111533\"}]]]></receivers>";
    	str = xml1 + str + xml2;
    	Map<String,Object> resultMap = WxXmlParser.parseXML(str);
    	System.out.println(resultMap.toString());
    	System.out.println(resultMap.get("receivers"));
    	List<Receivers> listReceivers = JSONObject.parseArray(resultMap.get("receivers").toString(), Receivers.class);
    	for (Receivers receivers : listReceivers) {
			System.out.println(receivers.toString());
		}
    	String a = "20200603111533";
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        try {
			System.out.println(sdf.parse(a).getTime());
		} catch (ParseException e) {
			e.printStackTrace();
		}
    }



    private RespWrapper<KtjJsapiOrder> findOrder(SepUserHandleHelp handleHelp){
        KtjJsapiOrder order = new KtjJsapiOrder();
        order.setTransactionId(handleHelp.getTransactionId());//查询
        order = ktjJsapiOrderDao.templateOne(order);
        if(order == null) return RespWrapper.makeFailResp("微信支付订单号错误");
        if(!handleHelp.getSubMchId().equals(order.getSubMchid()))
            return RespWrapper.makeFailResp("请检查特约商户账号或微信支付订单号");
        if(order.getOrderStatus() == 0)
            return RespWrapper.makeFailResp("该订单还没有支付成功，无法分账");
        return RespWrapper.makeSuccessResp(order);
    }
    private static final String SEP_RECEIVERS = "{\"type\": \"%s\",\"account\":\"%s\",\"amount\":%s,\"description\": \"%s\"}";
    private RespWrapper<Map<String,Object>> handleSepAccount(String subMchId,String transactionId,String outOrderNo,String receivers,String subAppId){
        Map<String,Object> condition = new HashMap<>();
        condition.put("sub_appid", subAppId);//sub_mchId
        condition.put("sub_mch_id", subMchId);//sub_mchId
        condition.put("appid",ProjectConstant.appid);// appid
        condition.put("transaction_id",transactionId);
        condition.put("out_order_no",outOrderNo);
        condition.put("receivers", receivers);
        return doWxRequest(SepAccountEnums.单次分账,condition,true);
    }


    private RespWrapper<Map<String, Object>> delReceiver(KtjSepUser ktjSepUser, String subMchId) {
        Map<String,Object> condition = new HashMap<>();
        condition.put("sub_mch_id", subMchId);//sub_mchId
        condition.put("appid",ProjectConstant.appid);// appid
        condition.put("receiver",String.format(DEL_SEP_ACCOUNT_USER,ktjSepUser.getType(),ktjSepUser.getAccount()));
        return doWxRequest(SepAccountEnums.删除分账接收方, condition, false);
    }

    private RespWrapper<Map<String, Object>> addReceiver(KtjSepUser user,String subMchId){
    	KtjMerchantDetails sys = new KtjMerchantDetails();
    	sys.setSubMchid(subMchId);
    	sys = merchantDetailsDao.templateOne(sys);
        Map<String,Object> condition = new HashMap<>();
        condition.put("sub_mch_id", subMchId);//sub_mchId
        condition.put("appid",ProjectConstant.appid);// appid
        condition.put("sub_appid", sys.getMiniProgramSubAppid());//sub_mchId
        condition.put("receiver",String.format(ADD_WX_ACCOUNT_USER,user.getType(),user.getAccount(),user.getName(),user.getRelationType()));
        return doWxRequest(SepAccountEnums.添加分账接收方, condition, false);
    }
    /**
     * 如果分组为空，则不需向微信删除分账人
     * 1、如果在数据库存在数据--> 在数据库新增一条记录
     * 2、如果没有在数据库--> 增加数据库的，并向微信请求新增
     */
    private void handleSepUserData(List<KtjSepUser> saveDBAndWx,List<KtjSepUser> saveDB,Map<String,KtjSepUser> accountMap,UserSepReceiver userSepReceiver,String groupName){
        for(SepReceiverHelp hr:userSepReceiver.getSepUserList()){
            KtjSepUser ktjSepUser = accountMap.get(hr.getWxAccount());
            // 第2种情况
            if(ktjSepUser == null){
                saveDBAndWx.add(initSepUser(hr,groupName,userSepReceiver.getSubMchId()));
            }else {
                // 第1种情况
                ktjSepUser.setGroupName(groupName);
                ktjSepUser.setSepRate(hr.getSepRate());
                ktjSepUser.setId(null);
                saveDB.add(ktjSepUser);
            }
        }
    }
    private KtjSepUser doInitSepUser(KtjSepUser ktjSepUser,SepReceiverHelp hr,String groupName,String subMchId){
        // 第2种情况
        if(ktjSepUser == null){
            return initSepUser(hr,groupName,subMchId);
        }else {
            // 第1种情况
            ktjSepUser.setGroupName(groupName);
            ktjSepUser.setSepRate(hr.getSepRate());
            ktjSepUser.setId(null);
        }
        return ktjSepUser;
    }




    // 获取其他分组的引用
    private Map<String, KtjSepUser> findWxAccountMap(List<KtjSepUser> template,String groupName) {
        Map<String, KtjSepUser> map  = new HashMap<>();
        template.forEach(s->{
            if(!groupName.equals(s.getGroupName()))
                map.put(s.getAccount(),s);
        });
        return map;
    }


    private List<KtjSepUser> findGroupSepUserMap(List<KtjSepUser> template,String groupName) {
        return template.stream().filter(s->s.getGroupName().equals(groupName)).collect(Collectors.toList());
    }


    private RespWrapper<Boolean> checkSepFunction(String subMchId){
        Integer registerStatus = sepUserDao.findMerchantRegisterStatus(subMchId);
        if(registerStatus == null) return RespWrapper.makeFailResp("该商户还没有入驻,请先入驻");
        if(registerStatus == 1) return RespWrapper.makeFailResp("该商户审核中，不能进行分账人设置");
        if(registerStatus == 3) return RespWrapper.makeFailResp("该商户入驻失败，请重新入驻");
        /*RegisterStatus status = RegisterStatus.findRegisterStatus(registerStatus);
        if(status == null) return RespWrapper.makeFailResp("该商户还没有入驻,请先入驻");
        if(status != RegisterStatus.APPLYMENT_STATE_FINISHED)
            return RespWrapper.makeFailResp(status.getStatus());*/
        return RespWrapper.makeSuccessResp(true);
    }


    /**
     * 包括商户号，签名类型、随机数
     * @param enums
     * @param condition
     * @param isSSL
     * @return
     */
    private RespWrapper<Map<String,Object>> doWxRequest(SepAccountEnums enums, Map<String,Object> condition, boolean isSSL){
        Assert.notNull(condition,"微信请求参数不能为空");
        condition.put("mch_id", ProjectConstant.merchantId);//商户id
        condition.put("sign_type","HMAC-SHA256");
        condition.put("nonce_str",ProjectConstant.findNonceStr());
        condition.put("sign",WeChatUtil.sha256_HMAC(condition, ProjectConstant.api));
        String xml = WeChatUtil.mapFormatXML(condition);
System.out.println("微信请求参数：" + xml);        
        try {
            String xmlStr;
            if(isSSL){
                xmlStr = ComtikaHttpsUtil.doCertWX(enums.getUrl(), xml);
            }else xmlStr = ComtikaHttpsUtil.doPost(enums.getUrl(), xml);
            Map<String,Object> resultMap = WxXmlParser.parseXML(xmlStr);   		//解析微信响应的XML结果
System.out.println("微信返回值：" + xmlStr);            
            if(checkNoticeSign(resultMap)){
                String return_code = (String) resultMap.get("return_code");
                if("SUCCESS".equals(return_code)){
                    String result_code = (String) resultMap.get("result_code");
                    if("SUCCESS".equals(result_code)){
                        return RespWrapper.makeSuccessResp(resultMap);
                    }
                    return RespWrapper.makeFailResp("err_code:"+resultMap.get("err_code")+",err_code_des:"+resultMap.get("err_code_des"));
                }
                return RespWrapper.makeFailResp((String) resultMap.get("return_msg"));
            }
            return RespWrapper.makeFailResp("校验签名错误");
        }catch (Exception e){
            logger.error("请求微信异常：{}",e);
            return RespWrapper.makeFailResp("请求异常,请稍后再试!异常："+e.getMessage());
        }
    }




    @Override
    public void destroy() throws Exception {
        ComtikaHttpsUtil.stop();
    }




    /**
     * 校验微信响应数据
      */
    private static boolean checkNoticeSign(Map<String,Object> params){
        String sign = (String) params.get("sign");
        if(sign == null) return false;
        return sign.equals(WeChatUtil.sha256_HMAC(params, ProjectConstant.api));
    }

    private static final String SEP_OUT_ORDER_NO = "KTJ_%s_%s_%s";
    private static String findSepOutOrderNo(){
        return findSepOutOrderNo("SEP");
    }
    private static String findSepOutOrderNo(String sep){
        String s = DateUtils.formatDate(new Date(), "yyyyMMdd");
        return String.format(SEP_OUT_ORDER_NO,sep,s,System.nanoTime());
    }

}
