package com.wechat.service.sepAccount.impl;

import com.alibaba.fastjson.JSONObject;
import com.wechat.common.enums.SepAccountEnums;
import com.wechat.common.parameter.ProjectConstant;
import com.wechat.common.response.RespWrapper;
import com.wechat.common.util.ComtikaHttpsUtil;
import com.wechat.common.util.WeChatUtil;
import com.wechat.common.util.WxXmlParser;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 分账测试类
 */
public class SepAccountTest {
    // 删除分账
    private static final String DEL_SEP_ACCOUNT_USER = "{\"type\":\"%s\",\"account\":\"%s\"}";
    // 添加分账
    private static final String WX_ACCOUNT_STRING = "{\"type\": \"%s\",\"account\": \"%s\",\"name\": \"%s\",\"relation_type\": \"%s\"} ";
    private static Logger logger = LoggerFactory.getLogger(SepAccountTest.class);
    // 添加分账人接了测试
    public static void main(String[] args)throws Exception{
        //addReceiver();
        // 单次分账
        //singleSepAccount();
        // 查询分账
        findSepAccountResult();
        // 删除分账人
        //delSepAccountUser();
        // 不分账接口
        //noSepAccount();

    }

   /* *
     *  分账接收方与原请求不一致
     *  多次分账后完结分账时，out_order_no 需要注意
     *  第一次是系统内部生成的，第二、第三次 是上次分账返回的order_id
     * @throws Exception*/

    private static void noSepAccount()throws Exception {
        Map<String,Object> condition = new HashMap<>();
        condition.put("mch_id", ProjectConstant.merchantId);//商户id
        condition.put("sub_mch_id", "1584278251");//sub_mchId
        condition.put("appid",ProjectConstant.appid);// appid
        condition.put("nonce_str",ProjectConstant.findNonceStr());
        condition.put("transaction_id","4200000533202004184533522457");
        condition.put("sign_type","HMAC-SHA256");
        condition.put("out_order_no","30000110062020041800341490392");
        condition.put("description","分账完成");
        condition.put("sign", WeChatUtil.sha256_HMAC(condition, ProjectConstant.api));
        String xml = WeChatUtil.mapFormatXML(condition);
        String s = ComtikaHttpsUtil.doCertWX("https://api.mch.weixin.qq.com/secapi/pay/profitsharingfinish", xml,ProjectConstant.merchantId);
        System.out.println(s);

    }

   /* *
     * 包括商户号，签名类型、随机数
     * @param enums
     * @param condition
     * @param isSSL
     * @return*/

    private RespWrapper<Map<String,Object>> doWxRequest(SepAccountEnums enums, Map<String,Object> condition, boolean isSSL){
        Assert.notNull(condition,"微信请求参数不能为空");
        condition.put("mch_id", ProjectConstant.merchantId);//商户id
        condition.put("sign_type","HMAC-SHA256");
        condition.put("nonce_str",ProjectConstant.findNonceStr());
        condition.put("sign",WeChatUtil.sha256_HMAC(condition, ProjectConstant.api));
        String xml = WeChatUtil.mapFormatXML(condition);
        try {
            String xmlStr;
            if(isSSL){
                xmlStr = ComtikaHttpsUtil.doCertWX(enums.getUrl(), xml);
            }else xmlStr = WeChatUtil.postWX(enums.getUrl(), xml);
            Map<String,Object> resultMap = WxXmlParser.parseXML(xmlStr);   		//解析微信响应的XML结果
            if(checkNoticeSign(resultMap))
                return RespWrapper.makeSuccessResp(resultMap);
            return RespWrapper.makeFailResp("校验签名错误");
        }catch (Exception e){
            logger.error("请求微信异常：{}",e);
            return RespWrapper.makeFailResp("请求异常,请稍后再试!异常："+e.getMessage());
        }
    }



  /*  *
     * {nonce_str=ed95b1c7fbd32fe8, receiver={"type":"PERSONAL_WECHATID","account":"wxid_haxp101y5saj21"}, appid=wxec504e0378010e16, sign=961EA0F51E9AC69229769F041DB5C05E86693F973F71B6DC0EF71D18709340D8, result_code=SUCCESS, mch_id=1503800091, sub_mch_id=1584278251, return_code=SUCCESS}
     */
    private static void delSepAccountUser() {
        Map<String,Object> condition = new HashMap<>();
        condition.put("mch_id", ProjectConstant.merchantId);//商户id
        condition.put("sub_mch_id", "1584278251");//sub_mchId
        condition.put("appid",ProjectConstant.appid);// appid
        condition.put("nonce_str",ProjectConstant.findNonceStr());
        condition.put("receiver",String.format(DEL_SEP_ACCOUNT_USER,"PERSONAL_WECHATID","wxid_haxp101y5saj21"));
        condition.put("sign_type","HMAC-SHA256");
        condition.put("sign",WeChatUtil.sha256_HMAC(condition, ProjectConstant.api));
        String xml = WeChatUtil.mapFormatXML(condition);
        String xmlStr = WeChatUtil.postWX("https://api.mch.weixin.qq.com/pay/profitsharingremovereceiver", xml);
        System.out.println(xmlStr);
        Map<String,Object> resultMap = WxXmlParser.parseXML(xmlStr);   		//解析微信响应的XML结果
        System.out.println(resultMap);
        System.out.println(checkNoticeSign(resultMap));

    }
/*
    *
     * 查询分账结果
     *
     **/

    private static void findSepAccountResult()throws Exception {
        Map<String,Object> condition = new HashMap<>();
        condition.put("mch_id", ProjectConstant.merchantId);//商户id
        condition.put("sub_mch_id", "1584278251");//sub_mchId
        //condition.put("appid",ProjectConstant.appid);// appid
        condition.put("nonce_str",ProjectConstant.findNonceStr());
        condition.put("transaction_id","4200000533202004184533522457");
        condition.put("sign_type","HMAC-SHA256");
        condition.put("out_order_no",outOrderNo);
        condition.put("sign",WeChatUtil.sha256_HMAC(condition, ProjectConstant.api));
        String xml = WeChatUtil.mapFormatXML(condition);
      /*  String xmlStr = WeChatUtil.postWX("https://api.mch.weixin.qq.com/pay/profitsharingquery", xml);
        System.out.println(xmlStr);*/
        String xmlStr = ComtikaHttpsUtil.doPost("https://api.mch.weixin.qq.com/pay/profitsharingquery", xml);
        Map<String,Object> resultMap = WxXmlParser.parseXML(xmlStr);   		//解析微信响应的XML结果
        System.out.println(resultMap);
        System.out.println(checkNoticeSign(resultMap));
    }
   /* *
     *   {
     *          "type": "MERCHANT_ID",
     *          "account":"190001001",
     *          "amount":100,
     *          "description": "分到商户"
     * },*/

    static class SepUser{
        String type;
        String account;
        int amount;
        String description;
        public SepUser(String type,String account,int amount,String description){
            this.type = type;
            this.account = account;
            this.amount = amount;
            this.description = description;
        }
        public String getType() {
            return type;
        }

        public void setType(String type) {
            this.type = type;
        }

        public String getAccount() {
            return account;
        }

        public void setAccount(String account) {
            this.account = account;
        }

        public int getAmount() {
            return amount;
        }

        public void setAmount(int amount) {
            this.amount = amount;
        }

        public String getDescription() {
            return description;
        }

        public void setDescription(String description) {
            this.description = description;
        }
    }
    /**
     * 单个分账操作
     */
    private static final String outOrderNo = "ktjSep20200418001";
/*
    *
     * <xml>
     * <return_code><![CDATA[SUCCESS]]></return_code>
     * <result_code><![CDATA[SUCCESS]]></result_code>
     * <mch_id><![CDATA[1503800091]]></mch_id>
     * <sub_mch_id><![CDATA[1584278251]]></sub_mch_id>
     * <appid><![CDATA[wxec504e0378010e16]]></appid>
     * <nonce_str><![CDATA[47f68c677907aa1b]]></nonce_str>
     * <sign><![CDATA[71B1A4DFEB6B835BB3676F25A61772B885ADD64BB9D969F29E0169BC0EE1D426]]></sign>
     * <transaction_id><![CDATA[4200000533202004184533522457]]></transaction_id>
     * <out_order_no><![CDATA[ktjSep20200418001]]></out_order_no>
     * <order_id><![CDATA[30000110062020041800341490392]]></order_id>
     * </xml>
     * @throws Exception*/

    private static void singleSepAccount() throws Exception{
        Map<String,Object> condition = new HashMap<>();
        condition.put("mch_id", ProjectConstant.merchantId);//商户id
        condition.put("sub_mch_id", "1584278251");//sub_mchId
        condition.put("appid",ProjectConstant.appid);// appid
        condition.put("nonce_str",ProjectConstant.findNonceStr());
        condition.put("transaction_id","4200000533202004184533522457");
        condition.put("sign_type","HMAC-SHA256");
        //condition.put("out_order_no",outOrderNo);
        condition.put("out_order_no","ktjSep20200418002");
        List<SepUser> list = new ArrayList<>();
        list.add(new SepUser("PERSONAL_WECHATID","wxid_haxp101y5saj21",1,"分到个人"));
        condition.put("receivers", JSONObject.toJSONString(list));
        condition.put("sign",WeChatUtil.sha256_HMAC(condition, ProjectConstant.api));
        String xml = WeChatUtil.mapFormatXML(condition);
        //String xmlStr = WeChatUtil.postWX("https://api.mch.weixin.qq.com/secapi/pay/profitsharing", xml);
        String s = ComtikaHttpsUtil.doCertWX("https://api.mch.weixin.qq.com/secapi/pay/profitsharing", xml,ProjectConstant.merchantId);
        System.out.println(s);
        //Map<String,Object> resultMap = WxXmlParser.parseXML(xmlStr);   		//解析微信响应的XML结果
        //System.out.println(resultMap);
        //System.out.println(checkNoticeSign(resultMap));

    }

  /*  *
     * 校验微信响应数据
     */
    private static boolean checkNoticeSign(Map<String,Object> params){
        String sign = (String) params.get("sign");
        if(sign == null) return false;
        return sign.equals(WeChatUtil.sha256_HMAC(params, ProjectConstant.api));
    }

    private static String account = "{ \n" +
            "       \"type\": \"PERSONAL_WECHATID\", \n" +
            "       \"account\": \"wxid_haxp101y5saj21\", \n" +
            "       \"name\": \"赵路羊\", \n" +
            "\"relation_type\": \"PARTNER\"\n" +
            "} ";

    // 添加分账人成功
    // {nonce_str=23b0e1633fae5d5c, receiver={"type":"PERSONAL_WECHATID","account":"wxid_haxp101y5saj21","relation_type":"PARTNER"}, appid=wxec504e0378010e16, sign=3E2424E858AD437D86D9B444AD5592F5215BCCD4B1A50E6276226B747A4A8B39, result_code=SUCCESS, mch_id=1503800091, sub_mch_id=1584278251, return_code=SUCCESS}
    private static void addReceiver(){
        Map<String,Object> condition = new HashMap<>();
        condition.put("mch_id", ProjectConstant.merchantId);//商户id
        condition.put("sub_mch_id", "1584278251");//sub_mchId
        condition.put("appid",ProjectConstant.appid);// appid
        condition.put("nonce_str",ProjectConstant.findNonceStr());
        condition.put("receiver",account);
        condition.put("sign_type","HMAC-SHA256");
        // 拼接签名
        String sign = WeChatUtil.sha256_HMAC(condition, ProjectConstant.api);
        condition.put("sign",sign);
        // 使用 HMAC-SHA256 签名
        String postXmlString = WeChatUtil.mapFormatXML(condition);// xml
        String xmlStr = WeChatUtil.postWX("https://api.mch.weixin.qq.com/pay/profitsharingaddreceiver", postXmlString);
        // 将xml转map
        Map<String,Object> resultMap = WxXmlParser.parseXML(xmlStr);   		//解析微信响应的XML结果
        System.out.println(resultMap);
        System.out.println(checkNoticeSign(resultMap));


    }
}
