package com.wechat.service.upload.impl;

import com.alibaba.fastjson.JSONObject;
import com.wechat.common.entity.KtjUploadBase;
import com.wechat.common.response.RespWrapper;
import com.wechat.common.util.WeChatUploadUtil;
import com.wechat.config.FtpConfig;
import com.wechat.core.ftp.FtpUtils;
import com.wechat.dao.UploadBaseDao;
import com.wechat.service.upload.IUploadService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.util.UUID;

/**
 * 上传模块
 */
@Service
public class UploadServiceImpl implements IUploadService {
    private static final String FILE_PATH_WX = "/mch/weChat/";
    private static final String FILE_PATH_BASE = "/mch/base/";

    @Autowired
    private UploadBaseDao uploadBaseDao;
    @Autowired
    private FtpConfig ftpConfig;

    @Override
    public RespWrapper<JSONObject> weChatUpload(MultipartFile file) {
        return doUpload(file,true);
    }

    @Override
    public RespWrapper<JSONObject> baseUpload(MultipartFile file) {
        return doUpload(file,false);
    }
    private RespWrapper<JSONObject> doUpload(MultipartFile file, boolean isWeChat) {
        String originalFilename = file.getOriginalFilename();
        FtpUtils ftpUtils = null;
        try {
            byte[] bytes = null;//文件bytes
            KtjUploadBase base = new KtjUploadBase();
            if(isWeChat){
                bytes = file.getBytes();
                RespWrapper<String> wxUpload = WeChatUploadUtil.doWeChantPoto(bytes, originalFilename);
                if(!wxUpload.isSuccess())return RespWrapper.makeFailResp(wxUpload.getMsg());
                base.setMediaId(wxUpload.getData());
            }
            // 下面处理本地数据库的数据
            WXFile wxFile = new WXFile(originalFilename);
            ftpUtils = new FtpUtils(ftpConfig);
            String filePath = isWeChat?FILE_PATH_WX:FILE_PATH_BASE;
            if(isWeChat && bytes != null){
                // 这里需要根据byte 转inputStream
                ftpUtils.uploadFile(filePath,wxFile.newFileName,new ByteArrayInputStream(bytes));
            }else {
                ftpUtils.uploadFile(filePath,wxFile.newFileName,file.getInputStream());
            }
            base.setFileName(wxFile.newFileName);
            base.setCreateTime(System.currentTimeMillis());
            base.setFilePath(filePath);
            base.setOrgName(wxFile.fileName);
            String baseUrl = ftpConfig.getUploadUrl().concat(filePath).concat(wxFile.newFileName);
            base.setBaseUrl(baseUrl);
            uploadBaseDao.insert(base,true);
            JSONObject jsonObject = new JSONObject(4);
            jsonObject.put("id",base.getId());
            jsonObject.put("url",baseUrl);
            jsonObject.put("mediaId", base.getMediaId());
            return RespWrapper.makeSuccessResp(jsonObject);
        }catch (Exception e){
            e.printStackTrace();
            return RespWrapper.makeFailResp("异常:"+e.getMessage());
        }finally {
            if(ftpUtils != null)
                ftpUtils.close();
        }
    }

    //原始名字处理，原始名字不要太长
    private class WXFile{
        String fileName;//处理的原始名字
        String newFileName;//新的文件名字
        public WXFile(String orgFileName){
            int i = orgFileName.lastIndexOf(".");
            String suffix = orgFileName.substring(i);
            if(i > 15){
                StringBuilder sb = new StringBuilder();
                sb.append(orgFileName.substring(0,10));
                sb.append("...");
                sb.append(orgFileName.substring(i-2,i));
                sb.append(suffix);
                fileName = sb.toString();
            }else {
                fileName = orgFileName;
            }
            // 最后处理下新的名字
            newFileName = UUID.randomUUID().toString().replaceAll("-", "").toUpperCase().substring(16).concat(suffix);
        }
    }


}
